@@ -138,6 +138,14 @@ setMethod("crs<-", signature("SpatRaster", "ANY"),
 	}
 )
 
+setMethod("set.crs", signature("SpatRaster"), 
+	function(x, value) {
+		value <- .txtCRS(value)
+		x@ptr$set_crs(value)
+		messages(x, "set_crs")
+	}
+)
+
 
 
 #setMethod("crs<-", signature("SpatRaster", "character"), 
@@ -162,6 +170,15 @@ setMethod("crs", signature("SpatVectorProxy"),
 	}
 )
 
+
+#setMethod("crs", signature("sf"), 
+#	function(x, proj=FALSE, describe=FALSE, parse=FALSE) {
+#		xcrs <- attr(x[[ attr(x, "sf_column") ]], "crs")$wkt
+#		x <- vect(cbind(0,0), crs=xcrs)
+#		.get_CRS(x, proj=proj, describe=describe, parse=parse)
+#	}
+#)
+
 setMethod("crs<-", signature("SpatVector", "ANY"), 
 	function(x, value) {
 		value <- .txtCRS(value)
@@ -171,6 +188,13 @@ setMethod("crs<-", signature("SpatVector", "ANY"),
 	}
 )
 
+setMethod("set.crs", signature("SpatVector"), 
+	function(x, value) {
+		value <- .txtCRS(value)
+		x@ptr$set_crs(value)
+		messages(x, "set_crs")
+	}
+)
 
 
 setMethod("is.lonlat", signature("SpatRaster"), 
