@@ -10,6 +10,9 @@
 # RH: adapted from raster to terra 
 # Jan 11, 2022
 
+# RH: new cpp method (moved to distance.R)
+# May 14, 2022
+
 
 .calcDist <- function(r, origin=NULL, omit=NULL, filename, ...) {
 	x <- rast(r)
@@ -111,8 +114,8 @@
 
 
 
-setMethod("gridDistance", signature(x="SpatRaster"), 
-function(x, origin=NULL, omit=NULL, chunk=FALSE, filename="", overwrite=FALSE, ...) {
+
+.oldGridDistance <- function(x, origin=NULL, omit=NULL, chunk=FALSE, filename="", overwrite=FALSE, ...) {
 
 	if (! hasValues(x) ) {
 		error("gridDistance", "SpatRaster has no cells values")
@@ -236,7 +239,7 @@ function(x, origin=NULL, omit=NULL, chunk=FALSE, filename="", overwrite=FALSE, .
 		writeStop(out)
 	}
 }
-)
+
 
 
 
