@@ -33,16 +33,25 @@ messages <- function(x, f="") {
 
 
 mem_info <- function(x, n=1) {
+	#print=TRUE
 	n <- max(0,n)
 	opt <- spatOptions()
 	opt$ncopies = n;
 	v <- x@ptr$mem_needs(opt)
+	memmin <- opt$memmin
+	memmax <- opt$memmax
 	#if (print) {
 		gb <- 1024^3 / 8  # 
 		cat("\n------------------------")
 		cat("\nMemory (GB) ")
 		cat("\n------------------------")
-		cat(paste("\navailable       :",  round(v[2] / gb, 2)))
+
+		cat(paste("\ncheck threshold :", opt$memmin / gb, " (memmin)"))
+		if (memmax > 0) {
+			cat(paste("\navailable       :",  round(v[2] / gb, 2), "(memmax)"))
+		} else {
+			cat(paste("\navailable       :",  round(v[2] / gb, 2)))
+		}
 		cat(paste0("\nallowed (", round(100* v[3]) , "%)   : ", round(v[3] * v[2] / gb, 2)))
 
 		cat(paste0("\nneeded (n=", n, ")   ", ifelse(n<10, " : ", ": "), round(v[1] / gb, 2)))
@@ -52,6 +61,7 @@ mem_info <- function(x, n=1) {
 		cat("\n------------------------\n")
 	#} 
 	names(v) <- c("needed", "available", "memfrac", "chunksize")
+	invisible(v)
 }
 
 
