@@ -4,7 +4,9 @@
 	Z <- as.matrix(x, wide=TRUE)
 	Z[is.nan(Z) | is.infinite(Z)] <- NA
 
-	z <- stats::na.omit(round(as.vector(Z), 12))
+# loss of precision
+#	z <- stats::na.omit(round(as.vector(Z), 12))
+	z <- stats::na.omit(as.vector(Z))
 	n <- length(z)
 	if (n == 0) {
 		out$values = FALSE
@@ -118,9 +120,9 @@
 	ilevels <- match(out$levels, out$cats[[1]])
 	out$leg$legend <- unique(na.omit(out$cats[ilevels, 2]))
 	if (!is.null(out$coltab)) {
-	# perhaps merge(z, cats, colors) intead for clarity
+	# perhaps merge(z, cats, colors) instead for clarity
 		out$levels <- out$levels[!is.na(ilevels)]
-		m <- na.omit(match(out$levels[ilevels], out$coltab[,1]))
+		m <- na.omit(match(out$cats[[1]][ilevels], out$coltab[,1]))
 		out$coltab <- out$coltab[m, ,drop=FALSE]
 		out$cols <- grDevices::rgb(out$coltab[,2], out$coltab[,3], out$coltab[,4], out$coltab[,5], maxColorValue=255)
 		i <- match(z, out$coltab[,1])
@@ -491,7 +493,9 @@ setMethod("plot", signature(x="SpatRaster", y="missing"),
 
 		if (x@ptr$rgb) {
 			i <- x@ptr$getRGB() + 1
-			plotRGB(x, i[1], i[2], i[3], maxcell=maxcell, ...)
+			if (missing(main)) main = ""			
+			if (is.null(mar)) mar = 0			
+			plotRGB(x, i[1], i[2], i[3], maxcell=maxcell, mar=mar, main=main, ...)
 			return(invisible())
 		}
 
