@@ -35,6 +35,7 @@ setMethod("dots", signature(x="SpatVector"),
 
 
 .plotLines <- function(x, out, lty=1, lwd=1, ...) {
+	if (nrow(x) == 0) return(out)
 	cols <- out$cols
 	if (is.null(cols)) cols = rep("black", length(x))
 
@@ -58,6 +59,7 @@ setMethod("dots", signature(x="SpatVector"),
 
 .plotPolygons <- function(x, out, lty=1, lwd=1, density=NULL, angle=45, ...) {
 
+	if (nrow(x) == 0) return(out)
 	g <- geom(x, df=TRUE)
 	g <- split(g, g[,1])
 	g <- lapply(g, function(y) split(y, y[,2]))
@@ -151,9 +153,11 @@ setMethod("dots", signature(x="SpatVector"),
 
 	if (isTRUE(out$legend_sort)) {
 		out$uv <- sort(out$uv)
+	} else {
+		out$uv <- out$uv[!is.na(out$uv)]
 	}
 	ucols <- .getCols(length(out$uv), out$cols, 1)
-
+	
 	i <- match(out$v, out$uv)
 	out$cols <- ucols
 	out$main_cols <- ucols[i]
@@ -313,7 +317,8 @@ setMethod("dots", signature(x="SpatVector"),
 	} else if (out$legend_type == "depends") {
 		if (nuq < 11) {
 			out <- .vect.legend.classes(out)
-		} else if (!is.numeric(out$uv) && (nuq < 21)) {
+		} else if (!is.numeric(out$uv)) {
+			#if (nuq < 21)
 			out <- .vect.legend.classes(out)
 		} else {
 			out <- .vect.legend.interval(out, dig.lab=dig.lab)
@@ -548,3 +553,9 @@ setMethod("plot", signature(x="SpatVector", y="missing"),
 	}
 )
 
+
+setMethod("plot", signature(x="SpatVectorProxy", y="missing"), 
+	function(x, y, ...)  {
+		plot(ext(x), ...)
+	}
+)
