@@ -1,6 +1,6 @@
 
 setMethod("rapp", signature(x="SpatRaster"), 
-function(x, first, last, fun, ..., allyrs=FALSE, fill=NA, clamp=FALSE, filename="", overwrite=FALSE, wopt=list()) {
+function(x, first, last, fun, ..., allyrs=FALSE, fill=NA, clamp=FALSE, circular=FALSE, filename="", overwrite=FALSE, wopt=list()) {
 
 	stopifnot(hasValues(x))
 	firstval <- lastval <- NA
@@ -42,15 +42,15 @@ function(x, first, last, fun, ..., allyrs=FALSE, fill=NA, clamp=FALSE, filename=
 			if (txtfun %in% .cpp_funs) {
 				opt <- spatOptions(filename, overwrite, wopt=wopt)
 				na.rm <- isTRUE(list(...)$na.rm)
-				x@ptr <- x@ptr$rapply(index@ptr, firstval, lastval, txtfun, clamp, na.rm, opt)
+				x@ptr <- x@ptr$rapply(index@ptr, firstval, lastval, txtfun, clamp, na.rm, circular, opt)
 				return(messages(x, "rapp"))
 			}
 		} 
 	}
 	out <- rast(x)
-	v <- x@ptr$rappvals(index@ptr, firstval, lastval, clamp, allyrs, fill, 0, 1)
+	v <- x@ptr$rappvals(index@ptr, firstval, lastval, clamp, allyrs, fill, 0, 1, circular)
 	v <- sapply(v, fun, ...)
-	if (is.list(v)) { error("rapp", "values returned do not have the same length for each cell") }
+	if (is.list(v)) { error("rapp", "values returned by 'fun' do not have the same length for each cell") }
 	nc <- ncol(out)
 	trans = FALSE
 	if (NCOL(v) == nc) {
@@ -63,7 +63,7 @@ function(x, first, last, fun, ..., allyrs=FALSE, fill=NA, clamp=FALSE, filename=
 	}
 	b <- writeStart(out, filename, overwrite, wopt=wopt, n=nlyr(x)*3)
 	for (i in 1:b$n) {
-		v <- x@ptr$rappvals(index@ptr, firstval, lastval, clamp, allyrs, fill, b$row[i]-1, b$nrows[i])
+		v <- x@ptr$rappvals(index@ptr, firstval, lastval, clamp, allyrs, fill, b$row[i]-1, b$nrows[i], circular)
 		v <- sapply(v, fun, ...)
 		if (trans) v = t(v)
 		writeValues(out, as.vector(v), b$row[i], b$nrows[i])
