@@ -5,46 +5,56 @@
 
 
 new_rast <- function(nrows=10, ncols=10, nlyrs=1, xmin=0, xmax=1, ymin=0, ymax=1, crs, extent, resolution, vals, names, time, units) {
-		if (missing(extent)) {
-			e <- c(xmin, xmax, ymin, ymax) 
-		} else {
-			e <- as.vector(extent)
-		}
-		if ((e[1] >= e[2]) || e[3] >= e[4]) {
-			error("rast,missing", "invalid extent")
-		}
 
-		if (missing(crs)) {
-			if (e[1] > -360.01 & e[2] < 360.01 & e[3] > -90.01 & e[4] < 90.01) {
-				crs <- "OGC:CRS84"
-			} else {
-				crs <- ""
+	ncols <- round(ncols)
+	if (ncols < 1) error("rast", "ncols < 1")
+	nrows <- round(nrows)
+	if (nrows < 1) error("rast", "nrows < 1")
+	
+	if (missing(extent)) {
+		e <- c(xmin, xmax, ymin, ymax) 
+	} else {
+		e <- as.vector(extent)
+	}
+	if ((e[1] >= e[2]) || e[3] >= e[4]) {
+		error("rast,missing", "invalid extent")
+	}
+	if (missing(crs)) {
+		if (e[1] > -360.01 & e[2] < 360.01 & e[3] > -90.01 & e[4] < 90.01) {
+			crs <- "OGC:CRS84"
+		} else {
+			crs <- ""
+		}
+	} else {
+		crs <- character_crs(crs, "rast")
+	}
+	#check_proj4_datum(crs)
+	
+	r <- methods::new("SpatRaster")
+	r@ptr <- SpatRaster$new(c(nrows, ncols, nlyrs), e, crs)
+	r <- messages(r, "rast")
+
+	if (!missing(resolution)) {
+		res(r) <- resolution
+	}
+	if (!missing(names)) {
+		names(r) <- names
+	}
+	if (!missing(vals)) {
+		if (length(vals) == 1) {
+			if (is.na(vals[1])) {
+				vals <- as.numeric(NA)
 			}
-		} else {
-			crs <- character_crs(crs, "rast")
 		}
-		#check_proj4_datum(crs)
-
-		r <- methods::new("SpatRaster")
-		r@ptr <- SpatRaster$new(c(nrows, ncols, nlyrs), e, crs)
-		r <- messages(r, "rast")
-
-		if (!missing(resolution)) {
-			res(r) <- resolution
-		}
-		if (!missing(names)) {
-			names(r) <- names
-		}
-		if (!missing(vals)) {
-			values(r) <- vals
-		}
-		if (!missing(time)) {
-			time(r) <- time
-		}
-		if (!missing(units)) {
-			time(r) <- units
-		}
-		r
+		values(r) <- vals
+	}
+	if (!missing(time)) {
+		time(r) <- time
+	}
+	if (!missing(units)) {
+		time(r) <- units
+	}
+	r
 }
 
 
@@ -90,8 +100,9 @@ setMethod("rast", signature(x="list"),
 		}
 		# start with an empty raster (alternatively use a deep copy)
 		out <- rast(x[[1]])
+		opt <- spatOptions()
 		for (i in 1:length(x)) {
-			out@ptr$addSource(x[[i]]@ptr, FALSE)
+			out@ptr$addSource(x[[i]]@ptr, FALSE, opt)
 		}
 		out <- messages(out, "rast")
 		lnms <- names(x)
@@ -168,6 +179,7 @@ setMethod("rast", signature(x="character"),
 		}
 		r <- methods::new("SpatRaster")
 		f <- .fullFilename(x)
+		f <- enc2utf8(f)
 		#subds <- subds[1]
 		if (is.null(opts)) opts <- ""[0]
 		if (length(subds) == 0) subds = 0
@@ -236,6 +248,9 @@ multi <- function(x, subds=0, xyz=c(1,2,3)) {
 
 setMethod("rast", signature(x="SpatRaster"),
 	function(x, nlyrs=nlyr(x), names, vals, keeptime=TRUE, keepunits=FALSE, props=FALSE) {
+		if (inherits(nlyrs, "SpatRaster")) {
+			error("rast", "use 'c()' to combine SpatRasters")
+		}
 		x@ptr <- x@ptr$geometry(nlyrs, props, keeptime, keepunits)
 		x <- messages(x, "rast")
 		if (!missing(names)) {
@@ -299,7 +314,15 @@ setMethod("rast", signature(x="array"),
 
 setMethod("rast", signature(x="ANY"),
 	function(x, ...) {
-		methods::as(x, "SpatRaster")
+		if (inherits(x, "sf")) {
+			out <- rast(ext(x), ...)
+			if (is.null(list(...)$crs)) {
+				crs(out) <- crs(x)
+			}
+			out
+		} else {
+			methods::as(x, "SpatRaster")
+		}
 	}
 )
 
@@ -363,7 +386,10 @@ setMethod("rast", signature(x="ANY"),
 	cells <- cellFromXY(r, xyz[,1:2])
 	if (d[2] > 2) {
 		names(r) <- ln[-c(1:2)]
-		v <- matrix(NA, nrow=ncell(r), ncol= nlyr(r))
+		v <- try( matrix(NA, nrow=ncell(r), ncol= nlyr(r)) )
+		if (inherits(v, "try-error")) {
+			error(paste("cannot make matrix with ", ncell(r), " rows"))
+		}
 		v[cells, ] <- xyz[, -c(1:2)]
 		values(r) <- v
 	}
@@ -380,7 +406,7 @@ setMethod("rast", signature(x="matrix"),
 			r <- .rastFromXYZ(x, crs=crs, digits=digits, extent=extent)
 		} else {
 			if (is.null(extent)) {
-				r <- rast(nrows=nrow(x), ncols=ncol(x), crs=crs, extent=ext(c(0, 1, 0, 1)))
+				r <- rast(nrows=nrow(x), ncols=ncol(x), extent=ext(c(0, ncol(x), 0, nrow(x))), crs=crs)
 			} else {
 				r <- rast(nrows=nrow(x), ncols=ncol(x), crs=crs, extent=extent)
 			}
@@ -394,7 +420,7 @@ setMethod("rast", signature(x="matrix"),
 setMethod("rast", signature(x="data.frame"),
 	function(x, type="xyz", crs="", digits=6, extent=NULL) {
 		if (type == "xyz") {
-			r <- .rastFromXYZ(x, crs=crs, digits=digits, extent=extent)
+			.rastFromXYZ(x, crs=crs, digits=digits, extent=extent)
 		} else {
 			rast(as.matrix(x), type=type, crs=crs, digits=digits, extent=extent)
 		}
