@@ -1,3 +1,7 @@
+# Author: Robert J. Hijmans
+# Date:  October 2018
+# Version 1.0
+# License GPL v3
 
 
 setMethod("$<-", "SpatRaster",  
@@ -19,7 +23,7 @@ setMethod("$<-", "SpatRaster",
 		if (is.na(i)) {
 			c(x, value)
 		} else if (nlyr(x) == 1) {
-			value
+			value$deepcopy()
 		} else if (i == 1) {
 			c(value, x[[2:nlyr(x)]])
 		} else if (i == nlyr(x)) {
@@ -116,15 +120,17 @@ setReplaceMethod("[", c("SpatRaster","numeric", "missing"),
 		if (!is.null(dim(value))) {
 			#x@ptr <- x@ptr$replaceValues(i, value, ncol(value))
 			stopifnot(ncol(value) == nlyr(x))
-			bylyr = TRUE
+			bylyr <- TRUE
 			if (inherits(value, "data.frame")) {
 				value <- as.matrix(value)
 			}
 			value <- as.vector(value)
 		} 
 
-		if (!x@ptr$replaceCellValues(i-1, value, bylyr, spatOptions())) {
-			messages(x)
+		x@ptr <- x@ptr$deepcopy()
+		opt <- spatOptions()
+		if (!x@ptr$replaceCellValues(i-1, value, bylyr, opt)) {
+			messages(x, "'[<-'")
 		} else {
 			x
 		}
@@ -132,6 +138,29 @@ setReplaceMethod("[", c("SpatRaster","numeric", "missing"),
 )
 
 
+setMethod("set.values", signature(x="SpatRaster"), 
+	function(x, cells, values)  {
+		if (missing(cells) && missing(values)) {
+			x@ptr$readAll()
+			return(invisible(TRUE));
+		}
+		bylyr = FALSE
+		if (!is.null(dim(values))) {
+			stopifnot(ncol(values) == nlyr(x))
+			bylyr = TRUE
+			if (inherits(values, "data.frame")) {
+				values <- as.matrix(values)
+			}
+			values <- as.vector(values)
+		} 
+		if (!x@ptr$replaceCellValues(cells-1, values, bylyr, spatOptions())) {
+			messages(x)
+		} else {
+			invisible(TRUE)
+		}
+	}
+)
+
 
 setReplaceMethod("[", c("SpatRaster", "numeric", "numeric"),
 	function(x, i, j, value) {
