@@ -118,6 +118,13 @@ setMethod ("show" , "SpatVector",
 		cat(" geometry    :", geomtype(object), "\n")
 		cat(" dimensions  : ", d[1], ", ", d[2], "  (geometries, attributes)\n", sep="" ) 
 		cat(" extent      : ", e[1], ", ", e[2], ", ", e[3], ", ", e[4], "  (xmin, xmax, ymin, ymax)\n", sep="")
+		if (object@ptr$source != "") {
+			if (object@ptr$layer != tools::file_path_sans_ext(basename(object@ptr$source))) {
+				cat(" source      : ", basename(object@ptr$source), " (", object@ptr$layer, ")\n", sep="")
+			} else {
+				cat(" source      : ", basename(object@ptr$source), "\n", sep="")
+			}
+		}
 		cat(" coord. ref. :", .name_or_proj4(object), "\n")
 		if (d[2] > 0) {
 			nr <- min(d[1], 3)
@@ -136,6 +143,11 @@ setMethod ("show" , "SpatVectorProxy",
 		cat(" geometry    :", geomtype(object), "\n")
 		cat(" dimensions  : ", d[1], ", ", d[2], "  (geometries, attributes)\n", sep="" ) 
 		cat(" extent      : ", e[1], ", ", e[2], ", ", e[3], ", ", e[4], "  (xmin, xmax, ymin, ymax)\n", sep="")
+		if (object@ptr$v$layer != tools::file_path_sans_ext(basename(object@ptr$v$source))) {
+			cat(" source      : ", basename(object@ptr$v$source), " (", object@ptr$v$layer, ")\n", sep="")
+		} else {
+			cat(" source      : ", basename(object@ptr$v$source), "\n", sep="")
+		}
 		cat(" coord. ref. :", .name_or_proj4(object), "\n")
 		dd <- get.data.frame(object)
 		printDF(dd, 0, TRUE)
@@ -282,16 +294,15 @@ setMethod ("show" , "SpatRaster",
 				}
 				isf <- is.factor(object)
 				if (any(isf)) {
-					lv <- levels(object)
 					for (i in 1:length(isf)) {
 						if (i > mnr) break
 						if (isf[i]) {
-							cats <- stats::na.omit(lv[[i]])
-							cats <- sort(cats[cats != ""])
-							cats <- sort(cats[cats != "NA"])
+							cats <- cats(object, i, TRUE)[[1]]
+							j <- match(c(minv[i], maxv[i]), cats[,1])
+							cats <- cats[j, 2]
 							if (length(cats) > 0) {
 								minv[i] <- cats[1]
-								maxv[i] <- cats[length(cats)]
+								maxv[i] <- cats[2]
 							}
 						} 
 					}
@@ -300,7 +311,7 @@ setMethod ("show" , "SpatRaster",
 				wln <- nchar(ln)
 				if (any(u8)) {
 					# for Chinese: wln <- wln + u8 * wln
-					w <- pmax(wln, nchar(minv), nchar(maxv), nchar(uts))
+					w <- pmax(wln, nchar(minv), nchar(maxv), nchar(uts), na.rm = TRUE)
 					m <- rbind(paste0(rep(" ", max(wln)), collapse=""), minv, maxv)
 					if (hasunits) m <- rbind(m, uts)
 					# a loop because "width" is not recycled by format
@@ -310,7 +321,7 @@ setMethod ("show" , "SpatRaster",
 						m[1,i] <- paste0(paste0(rep(" ", addsp), collapse=""), ln[i])
 					}
 				} else {
-					w <- pmax(wln, nchar(minv), nchar(maxv), nchar(uts))
+					w <- pmax(wln, nchar(minv), nchar(maxv), nchar(uts), na.rm = TRUE)
 					m <- rbind(ln, minv, maxv)
 					if (hasunits) m <- rbind(m, uts)
 					# a loop because "width" is not recycled by format
