@@ -14,7 +14,8 @@ setMethod("time", signature(x="SpatRaster"),
 		if (tstep == "seconds") {
 			strptime("1970-01-01", "%Y-%m-%d", tz = "UTC") + d
 		} else if (tstep == "days") {
-			as.Date("1970-01-01") + d
+			d <- strptime("1970-01-01", "%Y-%m-%d", tz = "UTC") + d
+			as.Date(d)
 		#} else if (tstep == "months") {
 		#} else if (tstep == "years") {
 		} else { # raw 
@@ -27,7 +28,7 @@ setMethod("time", signature(x="SpatRaster"),
 setMethod("time<-", signature(x="SpatRaster"), 
 	function(x, value)  {
 		if (inherits(value, "Date")) {
-			#value <- as.POSIXlt(value)
+			value <- as.POSIXlt(value)
 			tstep <- "days"
 		} else if (inherits(value, "POSIXt")) {
 			tstep <- "seconds"
@@ -87,7 +88,11 @@ setMethod("units", signature(x="SpatRaster"),
 
 setMethod("units<-", signature(x="SpatRaster"), 
 	function(x, value)  {
-		value <- as.character(value)
+		if (is.null(value) || all(is.na(value))) {
+			value <- ""
+		} else {
+			value <- as.character(value)
+		}
 		if (! x@ptr$set_units(value)) {
 			error("units<-", "cannot set these  values")
 		}
