@@ -26,7 +26,6 @@
 #include "geodesic.h"
 #include "recycle.h"
 
-
 double distance_lonlat(const double &lon1, const double &lat1, const double &lon2, const double &lat2) {
 	double a = 6378137.0;
 	double f = 1/298.257223563;
@@ -41,7 +40,7 @@ double distance_lonlat(const double &lon1, const double &lat1, const double &lon
 std::vector<double> distance_lonlat(std::vector<double> &lon1, std::vector<double> &lat1, std::vector<double> &lon2, std::vector<double> &lat2) {
 	double a = 6378137.0;
 	double f = 1/298.257223563;
-	
+
     recycle(lon1, lon2);
     recycle(lat1, lat2);
 	std::vector<double> r (lon1.size());
@@ -109,7 +108,7 @@ double toDeg(double &rad) {
 double direction_lonlat(double lon1, double lat1, double lon2, double lat2, bool degrees) {
 	double a = 6378137.0;
 	double f = 1/298.257223563;
-	
+
 	double s12, azi1, azi2;
 	struct geod_geodesic g;
 	geod_init(&g, a, f);
@@ -123,7 +122,7 @@ double direction_lonlat(double lon1, double lat1, double lon2, double lat2, bool
 std::vector<double> direction_lonlat(std::vector<double> lon1, std::vector<double> lat1, std::vector<double> lon2, std::vector<double> lat2, bool degrees) {
 	double a = 6378137.0;
 	double f = 1/298.257223563;
-	
+
 // lonlat1 and lonlat2 should have the same length
 
 	std::vector<double> azi1(lon1.size());
@@ -145,50 +144,44 @@ std::vector<double> direction_lonlat(std::vector<double> lon1, std::vector<doubl
 }
 
 
-std::vector<double> directionToNearest_lonlat(std::vector<double> lon1, std::vector<double> lat1, std::vector<double> lon2, std::vector<double> lat2, bool degrees, bool from) {
+void directionToNearest_lonlat(std::vector<double> &azi, const std::vector<double> &lon1, const std::vector<double> &lat1, const std::vector<double> &lon2, const std::vector<double> &lat2, bool& degrees, bool& from) {
 
 	double a = 6378137.0;
 	double f = 1/298.257223563;
-	
+
 	double azi1, azi2, s12, dist;
 	size_t n = lon1.size();
 	size_t m = lon2.size();
-	std::vector<double> azi(n);
+	
+	azi.resize(n, NAN);
 
 	struct geod_geodesic g;
 	geod_init(&g, a, f);
 
-	if (from) {
-		for (size_t i=0; i < n; i++) {
-			geod_inverse(&g, lat2[0], lon2[0], lat1[i], lon1[i], &dist, &azi1, &azi2);
-			azi[i] = azi1;
-			for (size_t j=1; j<m; j++) {
-				geod_inverse(&g, lat2[j], lon2[j], lat1[i], lon1[i], &s12, &azi1, &azi2);
-				if (s12 < dist) {
-					azi[i] = azi1;
-				}
-			}
-			if (!degrees) {
-				azi[i] = toRad(azi[i]);
+	for (size_t i=0; i < n; i++) {
+		if (std::isnan(lat1[i])) { 
+			azi[i] = NAN;
+			continue;
+		}	
+		geod_inverse(&g, lat1[i], lon1[i], lat2[0], lon2[0], &dist, &azi1, &azi2);
+		size_t minj=0;
+		azi[i] = azi1;
+		for (size_t j=1; j<m; j++) {
+			geod_inverse(&g, lat1[i], lon1[i], lat2[j], lon2[j], &s12, &azi1, &azi2);
+			if (s12 < dist) {
+				minj = j;
+				dist = s12;
+				azi[i] = azi1;
 			}
 		}
-
-	} else {
-		for (size_t i=0; i < n; i++) {
-			geod_inverse(&g, lat1[i], lon1[i], lat2[0], lon2[0], &dist, &azi1, &azi2);
-			azi[i] = azi1;
-			for (size_t j=1; j<m; j++) {
-				geod_inverse(&g, lat1[i], lon1[i], lat2[j], lon2[j], &s12, &azi1, &azi2);
-				if (s12 < dist) {
-					azi[i] = azi1;
-				}
-			}
-			if (!degrees) {
-				azi[i] = toRad(azi[i]);
-			}
+		if (from) {
+			geod_inverse(&g, lat2[minj], lon2[minj], lat1[i], lon1[i], &s12, &azi1, &azi2);
+			azi[i] = azi1;	
+		}
+		if (!degrees) {
+			azi[i] = toRad(azi[i]);
 		}
 	}
-  	return azi;
 }
 
 
@@ -197,7 +190,7 @@ double direction_plane(double x1, double y1, double x2, double y2, bool degrees)
 	double a;
 	double pi2 = M_PI * 2;
 	a = fmod(atan2( x2 - x1, y2 - y1), pi2);
-	a = (a < 0 ? a + pi2 : a );
+	a = a < 0 ? a + pi2 : a;
 	return (degrees ? toDeg(a) : a);
 }
 
@@ -216,40 +209,30 @@ std::vector<double> direction_plane(std::vector<double> x1, std::vector<double>
 
 
 
-std::vector<double> directionToNearest_plane(std::vector<double> x1, std::vector<double> y1, std::vector<double> x2, std::vector<double> y2, bool degrees, bool from) {
+void directionToNearest_plane(std::vector<double> &r, const std::vector<double> &x1, const std::vector<double> &y1, const std::vector<double> &x2, const std::vector<double> &y2, bool& degrees, bool &from) {
 	size_t n = x1.size();
 	size_t m = x2.size();
-	std::vector<double> r(n);
+	r.resize(n, NAN);
 	double d, mind;
 	size_t minj;
-	if (from) {
-		for (size_t i = 0; i < n; i++) {
-			mind = distance_plane(x1[i], y1[i], x2[0], y2[0]);
-			minj = 0;
-			for (size_t j = 1; j < m; j++) {
-				d = distance_plane(x1[i], y1[i], x2[j], y2[j]);
-				if (d < mind) {
-					mind = d;
-					minj = j;
-				}
+	for (size_t i = 0; i < n; i++) {
+		r[i] = NAN;
+		if (std::isnan(x1[i])) continue; // x2 must not be NAN
+		mind = distance_plane(x1[i], y1[i], x2[0], y2[0]);
+		minj = 0;
+		for (size_t j = 1; j < m; j++) {
+			d = distance_plane(x1[i], y1[i], x2[j], y2[j]);
+			if (d < mind) {
+				mind = d;
+				minj = j;
 			}
-			r[i] = direction_plane(x2[minj], y2[minj], x1[i], y1[i], degrees);
 		}
-	} else {
-		for (size_t i = 0; i < n; i++) {
-			mind = distance_plane(x1[i], y1[i], x2[0], y2[0]);
-			minj = 0;
-			for (size_t j = 1; j < m; j++) {
-				d = distance_plane(x1[i], y1[i], x2[j], y2[j]);
-				if (d < mind) {
-					mind = d;
-					minj = j;
-				}
-			}
+		if (from) {
+			r[i] = direction_plane(x2[minj], y2[minj], x1[i], y1[i], degrees);
+		} else {
 			r[i] = direction_plane(x1[i], y1[i], x2[minj], y2[minj], degrees);
 		}
 	}
-  	return r;
 }
 
 
@@ -264,7 +247,7 @@ std::vector<double> destpoint_lonlat(double longitude, double latitude, double
 	geod_init(&g, a, f);
 	double lat2, lon2, azi2;
 	geod_direct(&g, latitude, longitude, bearing, distance, &lat2, &lon2, &azi2);
-	std::vector<double> out = {lon2, lat2, azi2 };
+	std::vector<double> out = { lon2, lat2, azi2 };
 	return out;
 }
 
@@ -272,7 +255,7 @@ std::vector<double> destpoint_lonlat(double longitude, double latitude, double
 std::vector<std::vector<double> > destpoint_lonlat(const std::vector<double> &longitude, const std::vector<double> &latitude, const std::vector<double> &bearing, const std::vector<double> &distance) {
 	double a = 6378137.0;
 	double f = 1/298.257223563;
-	
+
 	struct geod_geodesic g;
 	geod_init(&g, a, f);
 	size_t n = longitude.size();
@@ -291,7 +274,7 @@ std::vector<std::vector<double> > destpoint_lonlat(const std::vector<double> &lo
 std::vector<std::vector<double> > destpoint_lonlat(const double &longitude, const double &latitude, const std::vector<double> &bearing, const double& distance, bool wrap) {
 	double a = 6378137.0;
 	double f = 1/298.257223563;
-	
+
 	struct geod_geodesic g;
 	geod_init(&g, a, f);
 	size_t n = bearing.size();
@@ -319,8 +302,8 @@ std::vector<std::vector<double> > destpoint_lonlat(const double &longitude, cons
 
 std::vector<double> destpoint_plane(double x, double y, double bearing, double distance) {
 	bearing = bearing * M_PI / 180;
-	x += distance * cos(bearing);
-	y += distance * sin(bearing);
+	x += distance * sin(bearing);
+	y += distance * cos(bearing);
 	std::vector<double> out = {x, y};
 	return(out);
 }
@@ -333,8 +316,8 @@ std::vector<std::vector<double> > destpoint_plane(std::vector<double>  x, std::v
 	double xd, yd, b;
 	for (size_t i=0; i < n; i++) {
 		b = bearing[i] * M_PI / 180;
-		xd = x[i] + distance[i] * cos(b);
-		yd = y[i] + distance[i] * sin(b);
+		xd = x[i] + distance[i] * sin(b);
+		yd = y[i] + distance[i] * cos(b);
 		out.push_back( {xd, yd });
 	}
 	return(out);
@@ -360,7 +343,7 @@ void distanceToNearest_lonlat(std::vector<double> &d, const std::vector<double>
 			if (s12 < d[i]) {
 				d[i] = s12;
 			}
-		}		
+		}
 	}
 }
 
@@ -384,7 +367,7 @@ void distanceCosineToNearest_lonlat(std::vector<double> &d, const std::vector<do
 			if (s12 < d[i]) {
 				d[i] = s12;
 			}
-		}		
+		}
 	}
 }
 
@@ -480,12 +463,12 @@ void nearest_lonlat_self(std::vector<long> &id, std::vector<double> &d, std::vec
 			nlat[i] = lat[0];
 			id[i] = 0;
 		} else {
-			geod_inverse(&g, lat[1], lon[1], lat[0], lon[0], &d[i], &azi1, &azi2);	
+			geod_inverse(&g, lat[1], lon[1], lat[0], lon[0], &d[i], &azi1, &azi2);
 			nlon[i] = lon[1];
 			nlat[i] = lat[1];
 			id[i] = 1;
 		}
-		
+
 		for (size_t j=1; j < n; j++) {
 			if (j == i) continue;
 			geod_inverse(&g, lat[i], lon[i], lat[j], lon[j], &s12, &azi1, &azi2);
@@ -495,7 +478,7 @@ void nearest_lonlat_self(std::vector<long> &id, std::vector<double> &d, std::vec
 				nlon[i] = lon[j];
 				nlat[i] = lat[j];
 			}
-		}		
+		}
 	}
 }
 
