@@ -40,7 +40,7 @@ std::vector<double> rcValue(std::vector<double> &d, const int& nrow, const int&
 
 
 
-std::vector<double> SpatRaster::focal_values(std::vector<unsigned> w, double fillvalue, int row, int nrows, SpatOptions &ops) {
+std::vector<double> SpatRaster::focal_values(std::vector<unsigned> w, double fillvalue, int_64 row, int_64 nrows, SpatOptions &ops) {
 
 	if (nlyr() > 1) {
 		std::vector<unsigned> lyr = {0};
@@ -53,58 +53,66 @@ std::vector<double> SpatRaster::focal_values(std::vector<unsigned> w, double fil
 		std::vector<double> d;
 		return(d);
 	}
+	const bool global = is_global_lonlat();
 
-	int wr = w[0] / 2;
-	int nr = nrow();
-	int nc = ncol();
-	wr = std::min(wr, nr-1);
+	int_64 nr = nrow();
+	nrows = std::min(nrows, nr - row + 1);
+	
+	int_64 nc = ncol();
+	int_64 wr = w[0] / 2;
+	int_64 wc = w[1] / 2;
+	//may be unexptected
+	//wr = std::min(wr, nr-1);
+	//wc = std::min(wc, nc-1);
 
-	int startrow = row-wr;
+	int_64 startrow = row-wr;
 	startrow = startrow < 0 ? 0 : startrow;
-	int startoff = row-startrow;
+	int_64 startoff = row-startrow;
 
-	int readnrows = nrows+startoff+wr;
-	int endoff = wr;
+	nrows = nrows < 1 ? 1 : nrows;
+	int_64 readnrows = nrows+startoff+wr;
+	int_64 endoff = wr;
 	if ((startrow+readnrows) > nr ) {
 		readnrows = nr-startrow;
 		endoff = readnrows - (nrows+startoff);
 	}
-	std::vector<double> d;
-	readValues(d, startrow, readnrows, 0, nc);
 
-//	get_focal(f, d, nrows, nc, w[0], w[1], offset, endoff, fillvalue);
-//  get_focal(std::vector<double> &out, const std::vector<double> &d, int nrow, int ncol, int wrows, int wcols, int startoff, int endoff,  double fill) {
-
-	int wc = w[1] / 2;
-	wr = std::min(wr, std::max(1, nrows-1));
-	wc = std::min(wc, nc-1);
+// ??
+	//wr = std::min(wr, std::max((int_64)1, nrows-1));
 
 	size_t n = nrows * nc * w[0] * w[1];
-	std::vector<double> out(n, fillvalue);
-	int nrmax = nrows + startoff + endoff - 1;
+	int_64 nrmax = nrows + startoff + endoff - 1;
 	//int nrmax = d.size() / ncol - 1;
-	int f = 0;
-	const bool global = is_global_lonlat();
+	size_t f = 0;
 
-	for (int r=0; r < nrows; r++) {
-		for (int c=0; c < nc; c++) {
-			for (int i = -wr; i <= wr; i++) {
-				int row = r+startoff+i;
+	std::vector<double> d;
+	readValues(d, startrow, readnrows, 0, nc);
+	std::vector<double> out(n, fillvalue);
+
+//Rcpp::Rcout << "sr " << startrow << " so " << startoff << " rnr " << readnrows << " wr " << wr << " wc " << wc << " nrows " << nrows << std::endl; 
+
+
+	for (int_64 r=0; r < nrows; r++) {
+		for (int_64 c=0; c < nc; c++) {
+			for (int_64 i = -wr; i <= wr; i++) {
+				int_64 row = r+startoff+i;
 				if ((row < 0) || (row > nrmax)) {
 					f += w[1];
 				} else {
-					unsigned bcell = row * nc;
-					for (int j = -wc; j <= wc; j++) {
-						int col = c + j;
+					size_t bcell = row * nc;
+					for (int_64 j = -wc; j <= wc; j++) {
+						int_64 col = c + j;
 						if ((col >= 0) && (col < nc)) {
-							out[f] = d[bcell+col];
+							size_t idx = bcell+col;
+							out[f] = d[idx];
 						} else if (global) {
 							if (col < 0) {
 								col = nc + col;
 							} else if (col >= nc) {
 								col = col - nc;
 							} 
-							out[f] = d[bcell+col]; 
+							size_t idx = bcell+col;
+							out[f] = d[idx]; 
 						}
 						f++;
 					}
@@ -457,39 +465,26 @@ SpatRaster SpatRaster::focal(std::vector<unsigned> w, std::vector<double> m, dou
 	}
 
 	std::vector<double> fill;
-	for (size_t i = 0; i < out.bs.n; i++) {
-		unsigned rstart, roff;
-		unsigned rnrows = out.bs.nrows[i];
-		if (i == 0) {
-			rstart = 0;
-			roff = dhw0;
-			if (i != (out.bs.n-1)) {
-				rnrows += hw0;
+	if (nl == 1) {
+		for (size_t i = 0; i < out.bs.n; i++) {
+			unsigned rstart, roff;
+			unsigned rnrows = out.bs.nrows[i];
+			if (i == 0) {
+				rstart = 0;
+				roff = dhw0;
+				if (i != (out.bs.n-1)) {
+					rnrows += hw0;
+				}
+			} else {
+				rstart = out.bs.row[i] + hw0;
+				roff = hw0;
+				if (i == (out.bs.n-1)) {
+					rnrows -= hw0; 
+				}
 			}
-		} else {
-			rstart = out.bs.row[i] + hw0;
-			roff = hw0;
-			if (i == (out.bs.n-1)) {
-				rnrows -= hw0; 
-			}
-		}
-
-		std::vector<double> vout, voutcomb;
-		std::vector<double> vin, vincomb;
-		size_t off=0;
-		if (nl > 1) {
-			readValues(vincomb, rstart, rnrows, 0, nc);
-			off = nc * out.bs.nrows[i];
-			voutcomb.reserve(vincomb.size());
-		} else {
+			std::vector<double> vout, vin;
 			readValues(vin, rstart, rnrows, 0, nc);
-		}
-		for (size_t lyr=0; lyr<nl; lyr++) {
 			vout.clear();
-			if (nl > 1) {
-				size_t lyroff = lyr * off;
-				vin = {vincomb.begin() + lyroff, vincomb.begin() + lyroff + off}; 
-			}
 			if (i==0) {
 				if (expand) {
 					fill = {vin.begin(), vin.begin()+nc};
@@ -513,7 +508,6 @@ SpatRaster SpatRaster::focal(std::vector<unsigned> w, std::vector<double> m, dou
 				}
 				vin.insert(vin.end(), fill.begin(), fill.end());
 			}
-
 			if (dofun) {
 				focal_win_fun(vin, vout, nc, roff, out.bs.nrows[i], m, w[0], w[1], fillvalue, narm, naonly, naomit, expand, global, fFun);
 			} else if (fun == "mean") {
@@ -525,26 +519,69 @@ SpatRaster SpatRaster::focal(std::vector<unsigned> w, std::vector<double> m, dou
 			if (i != (out.bs.n-1)) {
 				fill = {vin.end() - fsz2, vin.end() };  
 			}
-
-			if (naonly) {
-				for (size_t j=0; j<vout.size(); j++) {
-					size_t k = fsz2 + j;
-					if (!std::isnan(vin[k])) {
-						vout[j] = vin[k];
-					}
-				}
-			}
-			if (nl > 1) {
-				voutcomb.insert(voutcomb.end(), vout.begin(), vout.end());
-			}
-		}
-		if (nl > 1) {
-			if (!out.writeBlock(voutcomb, i)) return out;
-		} else {
 			if (!out.writeBlock(vout, i)) return out;
 		}
-	}
+	} else {
+		for (size_t i = 0; i < out.bs.n; i++) {
+			unsigned rstart, roff;
+			unsigned rnrows = out.bs.nrows[i];
+			if (i != (out.bs.n-1)) {
+				rnrows += hw0;
+			}
+			if (i == 0) {
+				rstart = 0;
+				roff = dhw0;
+			} else {
+				rstart = out.bs.row[i] - hw0;
+				roff = hw0;
+				rnrows += hw0; 
+			}
 
+			std::vector<double> vout, voutcomb, vin, vincomb;
+			size_t off=0;
+			readValues(vincomb, rstart, rnrows, 0, nc);
+			off = nc * rnrows; //out.bs.nrows[i];
+			voutcomb.reserve(vincomb.size());
+			for (size_t lyr=0; lyr<nl; lyr++) {
+				vout.clear();
+				size_t lyroff = lyr * off;
+				vin = {vincomb.begin() + lyroff, vincomb.begin() + lyroff + off}; 
+				if (i==0) {
+					if (expand) {
+						fill = {vin.begin(), vin.begin()+nc};
+						for (size_t i=1; i<dhw0; i++) {
+							fill.insert(fill.end(), fill.begin(), fill.end());
+						}
+					} else {
+						fill.resize(fsz2, fillvalue);
+					}
+					vin.insert(vin.begin(), fill.begin(), fill.end());
+				}
+
+				if (i == (out.bs.n-1)) {
+					if (expand) {
+						fill = {vin.end()-nc, vin.end()};
+						for (size_t i=1; i<dhw0; i++) {
+							fill.insert(fill.end(), fill.begin(), fill.end());
+						}
+					} else {
+						std::fill(fill.begin(), fill.end(), fillvalue);
+					}
+					vin.insert(vin.end(), fill.begin(), fill.end());
+				}
+
+				if (dofun) {
+					focal_win_fun(vin, vout, nc, roff, out.bs.nrows[i], m, w[0], w[1], fillvalue, narm, naonly, naomit, expand, global, fFun);
+				} else if (fun == "mean") {
+					focal_win_mean(vin, vout, nc, roff, out.bs.nrows[i], m, w[0], w[1], fillvalue, narm, naonly, naomit, expand, global);
+				} else {
+					focal_win_sum(vin, vout, nc, roff, out.bs.nrows[i], m, w[0], w[1], fillvalue, narm, naonly, naomit, expand, global);
+				}
+				voutcomb.insert(voutcomb.end(), vout.begin(), vout.end());
+			}
+			if (!out.writeBlock(voutcomb, i)) return out;
+		}
+	}
 	out.writeStop();
 	readStop();
 	return(out);
