@@ -72,7 +72,6 @@ std::string sectostr(int x) {
 
 GDALDataset* openGDAL(std::string filename, unsigned OpenFlag, std::vector<std::string> open_options) {
 
-	GDALDataset *poDataset;
 
 	//std::vector <char *> oo_char = string_to_charpnt(openopts); // open options
     //poDataset = static_cast<GDALDataset*>(GDALOpenEx( filename.c_str(), OpenFlag, NULL, oo_char.data(), NULL ));
@@ -84,12 +83,13 @@ GDALDataset* openGDAL(std::string filename, unsigned OpenFlag, std::vector<std::
 	for (size_t i=0; i<open_options.size(); i++) {
 		std::vector<std::string> opt = strsplit(open_options[i], "=");
 		if (opt.size() == 2) {
-//			Rcpp::Rcout << opt[0] << "=" << opt[1] << std::endl;	
+//			Rcpp::Rcout << opt[0] << "=" << opt[1] << std::endl;
 			openops = CSLSetNameValue(openops, opt[0].c_str(), opt[1].c_str());
 		}
 	}
 
-    poDataset = static_cast<GDALDataset*>(GDALOpenEx( filename.c_str(), OpenFlag, NULL, openops, NULL));
+	GDALDataset *poDataset = static_cast<GDALDataset*>(GDALOpenEx( filename.c_str(), OpenFlag, NULL, openops, NULL));
+	CSLDestroy(openops);
 	return poDataset;
 }
 
@@ -128,7 +128,7 @@ std::vector<std::string> get_metadata_sds(std::string filename) {
 		}
 	}
 	GDALClose( (GDALDatasetH) poDataset );
-	return meta;	
+	return meta;
 }
 
 std::vector<std::vector<std::string>> parse_metadata_sds(std::vector<std::string> meta) {
@@ -143,12 +143,14 @@ std::vector<std::vector<std::string>> parse_metadata_sds(std::vector<std::string
 		if (pos != std::string::npos) {
 			s.erase(0, pos + ndelim.length());
 			name.push_back(s);
-			std::string vdelim = "\":";
-			size_t pos = s.find(vdelim);
-
+			std::string vdelim = ":";
+			size_t pos = s.find_last_of(vdelim);
 			if (pos != std::string::npos) {
 				s.erase(0, pos + vdelim.length());
 				var.push_back(s);
+			} else {
+				std::string v = "v" + std::to_string(i);
+				var.push_back(v);		
 			}
 		} else {
 			size_t pos = s.find(ddelim);
@@ -156,22 +158,25 @@ std::vector<std::vector<std::string>> parse_metadata_sds(std::vector<std::string
 				s.erase(0, pos + ddelim.length());
 				pos = s.find("]");
 				std::string dims = s.substr(1, pos-1);
-			
+
 				std::vector<std::string> d = strsplit(dims, "x");
 				if (d.size() < 2) {
 					nl.push_back("0");
 					nr.push_back("0");
-					nc.push_back("0");	
+					nc.push_back("0");
 				} else if (d.size() == 2) {
 					nl.push_back("1");
 					nr.push_back(d[0]);
 					nc.push_back(d[1]);
 				} else {
 					size_t ds = d.size()-1;
-					size_t nls = stoi(d[ds-2]);
-					for (size_t i=0; i<(ds-2); i++) {
-						nls *= stoi(d[i]);
-					}
+					size_t nls = 0;
+					try {
+						nls = stoi(d[ds-2]);
+						for (size_t i=0; i<(ds-2); i++) {
+							nls *= stoi(d[i]);
+						}
+					} catch(...) {}
 					nl.push_back(std::to_string(nls));
 					nr.push_back(d[ds-1]);
 					nc.push_back(d[ds]);
@@ -181,14 +186,14 @@ std::vector<std::vector<std::string>> parse_metadata_sds(std::vector<std::string
 				pos = s.find(" ");
 				s = s.substr(0, pos);
 				desc.push_back(s);
-			
+
 
 			//	nr.push_back( std::to_string(sub.nrow()));
 			//	nc.push_back(std::to_string(sub.ncol()));
 			//	nl.push_back(std::to_string(sub.nlyr()));
 
 			} else {
-				desc.push_back("");			
+				desc.push_back("");
 			}
 		}
 	}
@@ -222,7 +227,7 @@ std::vector<std::vector<std::string>> sdinfo(std::string fname) {
 	if (metadata == NULL) {
 		out[0] = std::vector<std::string> {"no subdatasets"};
 		GDALClose( (GDALDatasetH) poDataset );
-		return out;	
+		return out;
 	}
 	std::vector<std::string> meta;
 	for (size_t i=0; metadata[i] != NULL; i++) {
@@ -233,6 +238,7 @@ std::vector<std::vector<std::string>> sdinfo(std::string fname) {
 		out[0] = std::vector<std::string> {"no subdatasets"};
 		return out;
 	}
+
 	SpatRaster sub;
 	std::vector<std::string> name, var, desc, nr, nc, nl;
 	std::string ndelim = "NAME=";
@@ -245,8 +251,8 @@ std::vector<std::vector<std::string>> sdinfo(std::string fname) {
 		if (pos != std::string::npos) {
 			s.erase(0, pos + ndelim.length());
 			name.push_back(s);
-			std::string vdelim = "\":";
-			size_t pos = s.find(vdelim);
+			std::string vdelim = ":";
+			size_t pos = s.find_last_of(vdelim);
 			if (sub.constructFromFile(s, {-1}, {""}, {})) {
 				nr.push_back( std::to_string(sub.nrow()));
 				nc.push_back(std::to_string(sub.ncol()));
@@ -255,6 +261,8 @@ std::vector<std::vector<std::string>> sdinfo(std::string fname) {
 			if (pos != std::string::npos) {
 				s.erase(0, pos + vdelim.length());
 				var.push_back(s);
+			} else {
+				var.push_back("v" + std::to_string(i));				
 			}
 		} else {
 			size_t pos = s.find(ddelim);
@@ -262,7 +270,7 @@ std::vector<std::vector<std::string>> sdinfo(std::string fname) {
 				s.erase(0, pos + ddelim.length());
 				desc.push_back(s);
 			} else {
-				desc.push_back("");			
+				desc.push_back("");
 			}
 		}
 	}
@@ -299,17 +307,17 @@ std::string gdalinfo(std::string filename, std::vector<std::string> options, std
 
 # include "gdal_utils.h" // requires >= 2.1
 
-SpatRaster SpatRaster::make_vrt(std::vector<std::string> filenames, SpatOptions &opt) {
+SpatRaster SpatRaster::make_vrt(std::vector<std::string> filenames, std::vector<std::string> options, SpatOptions &opt) {
 
 	SpatRaster out;
 	std::string outfile = opt.get_filename();
 	if (outfile == "") {
-		outfile = tempFile(opt.get_tempdir(), ".vrt");
-	} else if (file_exists(outfile) & (!opt.get_overwrite())) {
+		outfile = tempFile(opt.get_tempdir(), opt.pid, ".vrt");
+	} else if (file_exists(outfile) && (!opt.get_overwrite())) {
 		out.setError("output file exists. You can use 'overwrite=TRUE' to overwrite it");
 		return(out);
 	}
-	
+
 	std::vector<GDALDataset *> tiles;
 	std::vector<std::string> ops;
 
@@ -338,19 +346,25 @@ SpatRaster SpatRaster::make_vrt(std::vector<std::string> filenames, SpatOptions
             [-oo NAME=VALUE]*
 */
 
+	std::vector <char *> vops = string_to_charpnt(options);
+	GDALBuildVRTOptions* vrtops = GDALBuildVRTOptionsNew(vops.data(), NULL);
+	if (vrtops == NULL) {
+		out.setError("options error");
+		return(out);
+	}
 	int pbUsageError;
-	GDALDataset *ds = (GDALDataset *) GDALBuildVRT(outfile.c_str(), tiles.size(), (GDALDatasetH *) tiles.data(), nullptr, nullptr, &pbUsageError);
-//	GDALBuildVRTOptionsFree(vrtops);
+	GDALDataset *ds = (GDALDataset *) GDALBuildVRT(outfile.c_str(), tiles.size(), (GDALDatasetH *) tiles.data(), nullptr, vrtops, &pbUsageError);
+	GDALBuildVRTOptionsFree(vrtops);
 
 	for (size_t i= 0; i<tiles.size(); i++) GDALClose(tiles[i]);
 	if(ds == NULL )  {
-		out.setError("cannot create vrt. UsageError #"+ std::to_string(pbUsageError));
+		out.setError("cannot create vrt. Error #"+ std::to_string(pbUsageError));
 		return out;
 	}
 	GDALClose(ds);
 	if (!out.constructFromFile(outfile, {-1}, {""}, {})) {
 		out.setError("cannot open created vrt");
-		return out;		
+		return out;
 	}
 	return out;
 }
@@ -361,16 +375,24 @@ std::string gdalinfo(std::string filename, std::vector<std::string> options, std
 // adapted from the 'sf' package by Edzer Pebesma et al
 
 	std::string out = "";
-	std::vector <char *> options_char = string_to_charpnt(options);
-	std::vector <char *> oo_char = string_to_charpnt(openopts); // open options
-	GDALInfoOptions* opt = GDALInfoOptionsNew(options_char.data(), NULL);
-	GDALDatasetH ds = GDALOpenEx((const char *) filename.c_str(), GA_ReadOnly, NULL, oo_char.data(), NULL);
+	char ** opops = NULL;
+	for (size_t i=0; i<openopts.size(); i++) {
+		std::vector<std::string> opt = strsplit(openopts[i], "=");
+		if (opt.size() == 2) {
+			opops = CSLSetNameValue(opops, opt[0].c_str(), opt[1].c_str());
+		}
+	}
+	GDALDatasetH ds = GDALOpenEx(filename.c_str(), GA_ReadOnly, NULL, opops, NULL);
+	//if (opops != NULL) CSLDestroy(opops);
 	if (ds == NULL) return out;
+
+	std::vector <char *> options_char = string_to_charpnt(options);
+	GDALInfoOptions* opt = GDALInfoOptionsNew(options_char.data(), NULL);
 	char *val = GDALInfo(ds, opt);
 	out = val;
 	CPLFree(val);
-	GDALInfoOptionsFree(opt);
 	GDALClose(ds);
+	GDALInfoOptionsFree(opt);
 	return out;
 }
 
@@ -475,7 +497,7 @@ bool SpatRaster::as_gdalvrt(GDALDatasetH &hVRT, SpatOptions &opt) {
 			GDALAddBand(hVRT, GDT_Float64, papszOptions);
 		}
 	}
-	CSLDestroy(papszOptions);	
+	CSLDestroy(papszOptions);
 	return true;
 }
 
@@ -484,7 +506,7 @@ bool SpatRaster::as_gdalvrt(GDALDatasetH &hVRT, SpatOptions &opt) {
 bool SpatRaster::open_gdal(GDALDatasetH &hDS, int src, bool update, SpatOptions &opt) {
 
 	size_t isrc = src < 0 ? 0 : src;
-	
+
 	bool fromfile = !source[isrc].memory;
 
 	if (fromfile & (nsrc() > 1) & (src < 0)) {
@@ -517,16 +539,16 @@ bool SpatRaster::open_gdal(GDALDatasetH &hDS, int src, bool update, SpatOptions
 		/*
 			if (hDS != NULL) { // for user-set extents
 				std::vector<double> rs = resolution();
-				SpatExtent extent = getExtent();	
+				SpatExtent extent = getExtent();
 				double adfGeoTransform[6] = { extent.xmin, rs[0], 0, extent.ymax, 0, -1 * rs[1] };
 				GDALSetGeoTransform(hDS, adfGeoTransform);
 			}
-		*/	
+		*/
 		} else {
 			hDS = openGDAL(f, GDAL_OF_RASTER | GDAL_OF_READONLY | GDAL_OF_SHARED, source[src].open_ops);
 		}
 		return (hDS != NULL);
-	
+
 	} else { // in memory
 
 
@@ -536,7 +558,7 @@ bool SpatRaster::open_gdal(GDALDatasetH &hDS, int src, bool update, SpatOptions
 		if (src < 0) {
 			nl = nlyr();
 		} else {
-			nl = source[src].layers.size();		
+			nl = source[src].layers.size();
 		}
 		size_t ncls = nrow() * ncol();
 		GDALDriverH hDrv = GDALGetDriverByName("MEM");
@@ -564,7 +586,7 @@ bool SpatRaster::open_gdal(GDALDatasetH &hDS, int src, bool update, SpatOptions
 		if (hDS == NULL) return false;
 
 		std::vector<double> rs = resolution();
-		SpatExtent extent = getExtent();	
+		SpatExtent extent = getExtent();
 		double adfGeoTransform[6] = { extent.xmin, rs[0], 0, extent.ymax, 0, -1 * rs[1] };
 		GDALSetGeoTransform(hDS, adfGeoTransform);
 
@@ -574,16 +596,16 @@ bool SpatRaster::open_gdal(GDALDatasetH &hDS, int src, bool update, SpatOptions
 		}
 
 		CPLErr err = CE_None;
-	
+
 		if (hasval) {
 			std::vector<std::string> nms;
 			if (src < 0) {
 				nms = getNames();
 			} else {
-				nms = source[src].names;		
+				nms = source[src].names;
 			}
 
-			std::vector<double> vv, vals;	
+			std::vector<double> vv, vals;
 			if (src < 0) {
 				vv = getValues(-1, opt);
 			} else {
@@ -592,7 +614,7 @@ bool SpatRaster::open_gdal(GDALDatasetH &hDS, int src, bool update, SpatOptions
 					return false;
 				}
 			}
-	
+
 			for (size_t i=0; i < nl; i++) {
 				GDALRasterBandH hBand = GDALGetRasterBand(hDS, i+1);
 				GDALSetRasterNoDataValue(hBand, NAN);
@@ -668,9 +690,9 @@ bool SpatRaster::from_gdalMEM(GDALDatasetH hDS, bool set_geometry, bool get_valu
 			setError(msg);
 			return false;
 		}
-	
+
 		setSource(s);
-	
+
 	}
 
 	if (get_values) {
@@ -689,7 +711,7 @@ bool SpatRaster::from_gdalMEM(GDALDatasetH hDS, bool set_geometry, bool get_valu
 			}
 			//double naflag = -3.4e+38;
 			double naflag = GDALGetRasterNoDataValue(hBand, &hasNA);
-			if (hasNA && (!std::isnan(naflag))) {			
+			if (hasNA && (!std::isnan(naflag))) {
 				if (naflag < -3.4e+37) {
 					naflag = -3.4e+37;
 					for (size_t i=0; i<lyrout.size(); i++) {
@@ -713,7 +735,7 @@ bool SpatRaster::from_gdalMEM(GDALDatasetH hDS, bool set_geometry, bool get_valu
 			} else {
 				moffset = 0;
 			}
-			
+
 			double mscale = GDALGetRasterScale(hBand, &success);
 			if (success) {
 				if (mscale != 1) {
@@ -755,7 +777,7 @@ char ** set_GDAL_options(std::string driver, double diskNeeded, bool writeRGB, s
 			}
 		}
 		if (lzw) {
-			gdalops = CSLSetNameValue( gdalops, "COMPRESS", "LZW");			
+			gdalops = CSLSetNameValue( gdalops, "COMPRESS", "LZW");
 		}
 		if (compressed & (diskNeeded > 4194304000)) { 
 			bool big = true;
@@ -775,7 +797,7 @@ char ** set_GDAL_options(std::string driver, double diskNeeded, bool writeRGB, s
 	} else if (writeRGB) {
 		gdalops = CSLSetNameValue( gdalops, "PHOTOMETRIC", "RGB");
 	}
-	
+
 	for (size_t i=0; i<gdal_options.size(); i++) {
 		std::vector<std::string> gopt = strsplit(gdal_options[i], "=");
 		if (gopt.size() == 2) {
@@ -790,7 +812,7 @@ char ** set_GDAL_options(std::string driver, double diskNeeded, bool writeRGB, s
 bool SpatRaster::create_gdalDS(GDALDatasetH &hDS, std::string filename, std::string driver, bool fill, double fillvalue, std::vector<bool> has_so, std::vector<double> scale, std::vector<double> offset, SpatOptions& opt) {
 
 	has_so.resize(nlyr(), false);
-	
+
 	const char *pszFormat = driver.c_str();
 	GDALDriverH hDrv = GDALGetDriverByName(pszFormat);
 
@@ -809,7 +831,7 @@ bool SpatRaster::create_gdalDS(GDALDatasetH &hDS, std::string filename, std::str
 		GIntBig diskAvailable = VSIGetDiskFreeSpace(dname.c_str());
 		if ((diskAvailable > -1) && (diskAvailable < diskNeeded)) {
 			setError("insufficient disk space (perhaps from temporary files?)");
-			return(false);		
+			return(false);
 		}
 
 		papszOptions = set_GDAL_options(driver, diskNeeded, false, opt.gdal_options);
@@ -834,19 +856,19 @@ bool SpatRaster::create_gdalDS(GDALDatasetH &hDS, std::string filename, std::str
 
 	std::vector<std::string> nms = getNames();
 	std::vector<bool> hasCats = hasCategories();
-	
+
 	for (size_t i=0; i < nlyr(); i++) {
 		GDALRasterBandH hBand = GDALGetRasterBand(hDS, i+1);
 		GDALSetDescription(hBand, nms[i].c_str());
 		GDALSetRasterNoDataValue(hBand, naflag);
 		//GDALSetRasterNoDataValue(hBand, -3.4e+38);
 		if (fill) GDALFillRaster(hBand, fillvalue, 0);
-		
+
 		if (has_so[i]) {
 			GDALSetRasterOffset(hBand, offset[i]);
 			GDALSetRasterScale(hBand, scale[i]);
 		}
-		
+
 		if (hasCats[i]) {
 			std::vector<std::string> cats = getLabels(i);
 			char **names = NULL;
@@ -911,7 +933,7 @@ bool SpatRaster::setValuesMEM(GDALDatasetH hDS, bool set_geometry) {
 		s.driver = "memory";
 		s.names = source[0].names;
 		setSource(s);
-	
+
 		OGRSpatialReferenceH srs = GDALGetSpatialRef( hDS );
 		char *cp;
 		const char *options[3] = { "MULTILINE=YES", "FORMAT=WKT2", NULL };
@@ -942,12 +964,12 @@ bool SpatRaster::setValuesMEM(GDALDatasetH hDS, bool set_geometry) {
 			setError("CE_None");
 			return false;
 		}
-	
+
 		//double naflag = -3.4e+38;
 		double naflag = GDALGetRasterNoDataValue(hBand, &hasNA);
 		if (hasNA) std::replace(lyrout.begin(), lyrout.end(), naflag, (double) NAN);
 		source[0].values.insert(source[0].values.end(), lyrout.begin(), lyrout.end());
-	
+
 	}
 	source[0].hasValues = TRUE;
 	source[0].memory = TRUE;
@@ -1022,7 +1044,7 @@ bool SpatRaster::open_gdal(GDALDatasetH &hDS) {
 		std::string f = source[0].filename;
 		hDS = GDALOpen(f.c_str(), GA_ReadOnly);
 		return(hDS != NULL);
-	
+
 	} else { // in memory
 
 		size_t nl = nlyr();
@@ -1070,9 +1092,9 @@ bool SpatRaster::open_gdal(GDALDatasetH &hDS) {
 
 		CPLErr err = CE_None;
 		std::vector<double> vals;
-	
+
 		std::vector<std::string> nms = getNames();
-	
+
 		for (size_t i=0; i < nl; i++) {
 			GDALRasterBandH hBand = GDALGetRasterBand(hDS, i+1);
 			GDALSetRasterNoDataValue(hBand, NAN);
