@@ -1,28 +1,33 @@
 #define GEOS_USE_ONLY_R_API
 #include <geos_c.h>
 
-
 #if GEOS_VERSION_MAJOR == 3
 # if GEOS_VERSION_MINOR >= 5
-#  define HAVE350
+#  define GEOS350
 # endif
 # if GEOS_VERSION_MINOR == 6
 #  if GEOS_VERSION_PATCH >= 1
-#   define HAVE361
+#   define GEOS361
 #  endif
 # endif
 # if GEOS_VERSION_MINOR >= 7
-#  define HAVE361
-#  define HAVE370
+#  define GEOS361
+#  define GEOS370
+# endif
+# if GEOS_VERSION_MINOR >= 8
+#  define GEOS361
+#  define GEOS370
+#  define GEOS380
 # endif
 #else
 # if GEOS_VERSION_MAJOR > 3
-#  define HAVE350
-#  define HAVE370
-#  define HAVE361
+#  define GEOS350
+#  define GEOS370
+#  define GEOS361
 # endif
 #endif
 
+
 #include "spatVector.h"
 #include <cstdarg> 
 #include <cstring> 
@@ -112,7 +117,7 @@ static void __warningHandler(const char *fmt, ...) {
 
 
 void geos_finish(GEOSContextHandle_t ctxt) {
-#ifdef HAVE350
+#ifdef GEOS350
 	GEOS_finish_r(ctxt);
 #else
 	finishGEOS_r(ctxt);
@@ -121,7 +126,7 @@ void geos_finish(GEOSContextHandle_t ctxt) {
 
 
 GEOSContextHandle_t geos_init(void) {
-#ifdef HAVE350
+#ifdef GEOS350
 	GEOSContextHandle_t ctxt = GEOS_init_r();
 	GEOSContext_setNoticeHandler_r(ctxt, __warningHandler);
 	GEOSContext_setErrorHandler_r(ctxt, __errorHandler);
@@ -138,7 +143,7 @@ static void __warningIgnore(const char *fmt, ...) {
 
 GEOSContextHandle_t geos_init2(void) {
 
-#ifdef HAVE350
+#ifdef GEOS350
 	GEOSContextHandle_t ctxt = GEOS_init_r();
 	GEOSContext_setNoticeHandler_r(ctxt, __warningIgnore);
 	GEOSContext_setErrorHandler_r(ctxt, __errorHandler);
@@ -324,7 +329,7 @@ SpatVector vect_from_geos(std::vector<GeomPtr> &geoms , GEOSContextHandle_t hGEO
 	std::vector<double> x, y;
 	bool xok, yok;
 
-	if ((vt == "points") | (vt == "lines")) {	
+	if ((vt == "points") || (vt == "lines")) {	
 		for(size_t i = 0; i < ng; i++) {
 			GEOSGeometry* g = geoms[i].get();
 			size_t np = GEOSGetNumGeometries_r(hGEOSCtxt, g);
@@ -589,6 +594,7 @@ SpatVectorCollection coll_from_geos(std::vector<GeomPtr> &geoms, GEOSContextHand
 			//Rcpp::Rcout << np << std::endl;
 
 
+			size_t kk = 0; // introduced for intersect
 			for(size_t j = 0; j<np; j++) {
 
 				const GEOSGeometry* gg = GEOSGetGeometryN_r(hGEOSCtxt, g, j);
@@ -614,13 +620,12 @@ SpatVectorCollection coll_from_geos(std::vector<GeomPtr> &geoms, GEOSContextHand
 					if (increment) f++;
 				}
 
-	
 				for(size_t k = 0; k<npp; k++) {
 
 					const GEOSGeometry* part = GEOSGetGeometryN_r(hGEOSCtxt, gg, k);
 
 					if (ggt == "Polygon" || ggt == "MultiPolygon") {
-						if (!polysFromGeom(hGEOSCtxt, part, f, k, pl_x, pl_y, pl_gid, pl_gp, pl_hole, msg)) {
+						if (!polysFromGeom(hGEOSCtxt, part, f, kk, pl_x, pl_y, pl_gid, pl_gp, pl_hole, msg)) {
 							out.setError(msg);
 							return out;
 						}
@@ -643,10 +648,11 @@ SpatVectorCollection coll_from_geos(std::vector<GeomPtr> &geoms, GEOSContextHand
 					} else {
 						out.addWarning("unhandeled Collection geom: " + ggt);
 					}
-					if (increment) f++;
+					kk++;
 				}
-				if (!increment) f++;
+				if (increment) f++;
 			}
+			if (!increment) f++;
 		} else {
 			out.setError("what is this: " + gt + "?");
 		}
