@@ -16,6 +16,7 @@
 // along with spat. If not, see <http://www.gnu.org/licenses/>.
 
 
+#include <stdexcept>
 #include <algorithm>
 #include <stdint.h>
 #include <vector>
@@ -648,7 +649,8 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 			meta.push_back(metasds[i]);
 		}
 		GDALClose( (GDALDatasetH) poDataset );
-		return constructFromSDS(fname, meta, subds, subdsname, options, gdrv=="netCDF"); 
+		return constructFromSDS(fname, meta, subds, subdsname, options, gdrv); 
+		
 	} else if (nl==0) {
 		setError("no raster data in " + fname);
 		return false;
@@ -769,7 +771,7 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 	for (size_t i = 0; i < s.nlyr; i++) {
 		poBand = poDataset->GetRasterBand(i+1);
 
-		if ((gdrv=="netCDF") || (gdrv == "HDF5")) {
+		if ((gdrv=="netCDF") || (gdrv == "HDF5") || (gdrv == "GRIB")) {
 			char **m = poBand->GetMetadata();
 			while (*m != nullptr) {
 				bandmeta[i].push_back(*m++);
@@ -908,17 +910,19 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 	}
 
 
-	if ((gdrv=="netCDF") || (gdrv == "HDF5")) {
+	msg = "";
+	if ((gdrv=="netCDF") || (gdrv == "HDF5"))  {
 		std::vector<std::string> metadata;
 		char **m = poDataset->GetMetadata();
 		while (*m != nullptr) {
 			metadata.push_back(*m++);
 		}
-		std::string msg;
 		s.set_names_time_ncdf(metadata, bandmeta, msg);
-		if (msg.size() > 1) {
-			addWarning(msg);
-		}
+	} else if (gdrv == "GRIB") {
+		s.set_names_time_grib(bandmeta, msg);
+	}
+	if (msg.size() > 1) {
+		addWarning(msg);
 	}
 
 	GDALClose( (GDALDatasetH) poDataset );
@@ -1003,6 +1007,12 @@ void vflip(std::vector<double> &v, const size_t &ncell, const size_t &nrows, con
 
 void SpatRaster::readChunkGDAL(std::vector<double> &data, unsigned src, size_t row, unsigned nrows, size_t col, unsigned ncols) {
 
+
+	if (source[src].flipped) {
+		row = nrow() - row - nrows;
+	}
+
+
 	if (source[src].multidim) {
 		readValuesMulti(data, src, row, nrows, col, ncols);
 		return;
@@ -1090,6 +1100,9 @@ std::vector<double> SpatRaster::readValuesGDAL(unsigned src, size_t row, size_t
 		setError("cannot read from rotated files. First use 'rectify'");
 		return errout;
 	}
+	if (source[src].flipped) {
+		row = nrow() - row - nrows;
+	}
 
 	if (source[src].hasWindow) { // ignoring the expanded case.
 		row = row + source[src].window.off_row;
@@ -1116,7 +1129,7 @@ std::vector<double> SpatRaster::readValuesGDAL(unsigned src, size_t row, size_t
 		}
 	} else {
 		nl = 1;
-		panBandMap.push_back(lyr+1);
+		panBandMap.push_back(source[src].layers[lyr]+1);
 	}
 
 	std::vector<double> out(ncell*nl);
@@ -1439,10 +1452,18 @@ void ncdf_pick_most(std::vector<std::string> &sd, std::vector<std::string> &varn
 
 
 //todo: add open optionso
-bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string> meta, std::vector<int> subds, std::vector<std::string> subdsname, std::vector<std::string> options, bool ncdf) {
+bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string> meta, std::vector<int> subds, std::vector<std::string> subdsname, std::vector<std::string> options, std::string driver) {
+
+	bool ncdf = driver =="netCDF";
+	bool gtiff = driver == "GTiff";
 
 	std::vector<std::vector<std::string>> info = parse_metadata_sds(meta);
 	int n = info[0].size();
+
+	if (gtiff && (subds[0] < 0) && (subdsname[0] == "")) {
+		subds.resize(n);
+		std::iota(subds.begin(), subds.end(), 0);
+	}
 	std::vector<std::string> sd, varname, srcname;
 
 // std::vector<unsigned> varnl;
@@ -1452,7 +1473,7 @@ bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string>
 		return false;
 	}
 	// select sds by index
-	if ((subds.size() > 0) && (subds[0] >= 0)) {
+	if (((subds.size() > 0) && (subds[0] >= 0))) {
 		for (size_t i=0; i<subds.size(); i++) {
 			if (subds[i] >=0 && subds[i] < n) {
 				sd.push_back(info[0][subds[i]]);
@@ -1477,6 +1498,7 @@ bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string>
 				return false;
 			}
 		}
+		
 	// select all
 	} else {
 		// eliminate sources based on names like "*_bnds" and "lat"
@@ -1551,7 +1573,7 @@ bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string>
 		addWarning(s);
 	}
 
-	if (!ncdf) {
+	if (!(ncdf || gtiff)) {
 		std::vector<std::string> lyrnames;
 		for (size_t i=0; i<used.size(); i++) {
 			std::vector<std::string> nms = { basename(used[i]) };
@@ -1604,7 +1626,6 @@ bool get_double(std::string input, double &output) {
 
 std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vector<std::string> vals, std::string &step, std::string &msg) {
 
-
 	std::vector<int_64> out, bad;
 	if (vals.size() < 1) {
 		step = "";
@@ -1677,6 +1698,7 @@ std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vec
 		step = "seconds";
 		out.reserve(raw.size());
 		if (days) {
+			step = "days";
 			std::vector<int> ymd = getymd(origin);
 			if (calendar == "noleap" || calendar == "365_day" || calendar == "365 day") { 
 				for (size_t i=0; i<raw.size(); i++) out.push_back(time_from_day_noleap(ymd[0], ymd[1], ymd[2], raw[i]));
@@ -1767,10 +1789,8 @@ std::vector<std::vector<std::string>> ncdf_names(const std::vector<std::vector<s
 void SpatRasterSource::set_names_time_ncdf(std::vector<std::string> metadata, std::vector<std::vector<std::string>> bandmeta, std::string &msg) {
 
 	if (bandmeta.size() == 0) return;
-
 	std::vector<std::vector<std::string>> nms = ncdf_names(bandmeta);
 
-
 	if (nms[1].size() > 0) {
 		names = nms[1];
 		make_unique_names(names);
@@ -1785,18 +1805,99 @@ void SpatRasterSource::set_names_time_ncdf(std::vector<std::string> metadata, st
 	}
 
 	recycle(unit, nlyr);
-
-
 	if (nms[0].size() > 0) {
 		std::string step;
 		std::vector<int_64> x = ncdf_time(metadata, nms[0], step, msg);
-
 		if (x.size() == nlyr) {
 			time = x;
 			timestep = step;
 			hasTime = true;
 		}
+	}
+}
 
+
+
+
+std::vector<std::vector<std::string>> grib_names(const std::vector<std::vector<std::string>> &m) {
+
+	std::vector<std::vector<std::string>> out(3);
+	if (m.size() < 1) return out;
+
+	for (size_t i=0; i<m.size(); i++) {
+		std::string comm, time, units = "";
+		for (size_t j=0; j<m[i].size(); j++) {
+			size_t pos = m[i][j].find("GRIB_COMMENT=");
+			if (pos != std::string::npos) {
+				comm = m[i][j];
+				comm.erase(0, pos+13);
+				lrtrim(comm);
+				continue;
+			} 
+			pos = m[i][j].find("GRIB_UNIT=");
+			if (pos != std::string::npos) {
+				units = m[i][j];
+				units.erase(0, pos+10);
+				str_replace(units, "[", "");
+				str_replace(units, "]", "");
+				lrtrim(units);
+				continue;
+			} 
+			pos = m[i][j].find("TIME=");
+			if (pos != std::string::npos) {
+				std::string tmp = m[i][j];
+				tmp.erase(0, pos+5);
+				pos = tmp.find("sec");
+				if (pos != std::string::npos) {
+					tmp.erase(tmp.begin()+pos, tmp.end()); 					
+					time = tmp;
+				}
+				continue;
+			} 
+		}	
+		out[0].push_back(comm);
+		out[1].push_back(units);
+		out[2].push_back(time);
+	}	
+	return out;
+}
+
+
+
+void SpatRasterSource::set_names_time_grib(std::vector<std::vector<std::string>> bandmeta, std::string &msg) {
+
+	if (bandmeta.size() == 0) return;
+	std::vector<std::vector<std::string>> nms = grib_names(bandmeta);
+
+	if (nms[0].size() != names.size()) return;
+	
+	for (size_t i=0; i<names.size(); i++) {
+		names[i] += "; " + nms[0][i];
+		str_replace(names[i], "0[-] ", "");
+		str_replace_all(names[i], "\"", "");
+	}
+	unit = {nms[1]};
+
+	std::vector<int_64> tm;
+	bool hastime = true;
+	for (size_t i=0; i<nms[2].size(); i++) {
+		if (nms[2][i] == "") {
+			hastime = false;
+			break;
+		}
+		int_64 tim;
+		try {
+			tim = stol(nms[2][i]);
+		} catch(...) {
+			hastime = false;
+			break;
+		}
+		tm.push_back(tim);
+	}
+	if (hastime) {
+		time = tm;
+		timestep = "seconds";
+		hasTime = true;
 	}
 }
 
