@@ -64,7 +64,11 @@ SpatDataFrame readAttributes(OGRLayer *poLayer, bool as_proxy) {
 				if (ft == OFTReal) {
 					dtype = 0;
 				} else if ((ft == OFTInteger) | (ft == OFTInteger64)) {
-					dtype = 1;
+					if (poFieldDefn->GetSubType() == OFSTBoolean) {
+						dtype = 3;
+					} else {
+						dtype = 1;
+					}
 				} else {
 					dtype = 2;
 				}
@@ -81,7 +85,11 @@ SpatDataFrame readAttributes(OGRLayer *poLayer, bool as_proxy) {
 					df.dv[j].push_back(poFeature->GetFieldAsDouble(i));
 					break;
 				case OFTInteger:
-					df.iv[j].push_back(poFeature->GetFieldAsInteger( i ));
+					if (poFieldDefn->GetSubType() == OFSTBoolean) {
+						df.bv[j].push_back(poFeature->GetFieldAsInteger( i ));						
+					} else {
+						df.iv[j].push_back(poFeature->GetFieldAsInteger( i ));
+					}
 					break;
 				case OFTInteger64:
 					df.iv[j].push_back(poFeature->GetFieldAsInteger64( i ));
@@ -341,6 +349,15 @@ std::vector<std::string> SpatVector::layer_names(std::string filename) {
 	return out;
 }	
 
+SpatGeom emptyGeom() {
+	SpatGeom g;
+	g.gtype = null;
+	g.extent.xmin=NAN;
+	g.extent.xmax=NAN;
+	g.extent.ymin=NAN;
+	g.extent.ymax=NAN;
+	return g;
+}
 
 bool SpatVector::read_ogr(GDALDataset *poDS, std::string layer, std::string query, std::vector<double> extent, SpatVector filter, bool as_proxy) {
 
@@ -470,9 +487,10 @@ bool SpatVector::read_ogr(GDALDataset *poDS, std::string layer, std::string quer
 					g = getMultiPointGeom(poGeometry);
 				}
 			} else {
-				SpatPart p;
-				g = SpatGeom();
-				g.addPart(p);
+				//SpatPart p;
+				//g = SpatGeom();
+				//g.addPart(p);
+				g = emptyGeom();				
 			}
 			addGeom(g);
 			OGRFeature::DestroyFeature( poFeature );
@@ -485,9 +503,10 @@ bool SpatVector::read_ogr(GDALDataset *poDS, std::string layer, std::string quer
 					g = getMultiLinesGeom(poGeometry);
 				}
 			} else {
-				SpatPart p;
-				g = SpatGeom();
-				g.addPart(p);
+				//SpatPart p;
+				//g = SpatGeom();
+				//g.addPart(p);
+				g = emptyGeom();		
 			}
 			addGeom(g);
 			OGRFeature::DestroyFeature( poFeature );
@@ -501,7 +520,8 @@ bool SpatVector::read_ogr(GDALDataset *poDS, std::string layer, std::string quer
 					g = getMultiPolygonsGeom(poGeometry);
 				} 
 			} else {
-				g = SpatGeom();
+				//g = SpatGeom();
+				g = emptyGeom();
 			}
 			addGeom(g);
 			OGRFeature::DestroyFeature( poFeature );
@@ -532,9 +552,10 @@ bool SpatVector::read_ogr(GDALDataset *poDS, std::string layer, std::string quer
 					g = getMultiPointGeom(poGeometry);
 				}
 			} else {
-				SpatPart p;
-				g = SpatGeom();
-				g.addPart(p);
+				//SpatPart p;
+				//g = SpatGeom();
+				//g.addPart(p);
+				g = emptyGeom();			
 			}
 			addGeom(g);
 			OGRFeature::DestroyFeature( poFeature );
@@ -549,9 +570,10 @@ bool SpatVector::read_ogr(GDALDataset *poDS, std::string layer, std::string quer
 					g = getMultiLinesGeom(poGeometry);
 				}
 			} else {
-				SpatPart p;
-				g = SpatGeom();
-				g.addPart(p);
+				//SpatPart p;
+				//g = SpatGeom();
+				//g.addPart(p);
+				g = emptyGeom();		
 			}
 			addGeom(g);
 			OGRFeature::DestroyFeature( poFeature );
@@ -567,7 +589,8 @@ bool SpatVector::read_ogr(GDALDataset *poDS, std::string layer, std::string quer
 					g = getMultiPolygonsGeom(poGeometry);
 				} 
 			} else {
-				g = SpatGeom();
+				//g = SpatGeom();
+				g = emptyGeom();		
 			}
 			addGeom(g);
 			OGRFeature::DestroyFeature( poFeature );
