@@ -181,6 +181,16 @@ void SpatOptions::set_memmax(double d) {
 	} 
 }
 
+double SpatOptions::get_memmin() { return memmin; }
+
+void SpatOptions::set_memmin(double d) {
+	if (std::isnan(d) || (d <= 0)) {
+		memmin = 1024 * 1024 * 1024 / 8;
+	} else {
+		memmin = d * 1024 * 1024 * 1024 / 8;
+	} 
+}
+
 double SpatOptions::get_tolerance() { return tolerance; }
 
 void SpatOptions::set_tolerance(double d) {
@@ -303,7 +313,7 @@ void SpatRaster::setExtent(SpatExtent ext, bool keepRes, std::string snap) {
 
 	if (snap != "") {
 		ext = align(ext, snap);
-		ext.intersect(getExtent());
+		ext = ext.intersect(getExtent());
 	}
 
 	if (keepRes) {
@@ -437,7 +447,7 @@ bool SpatRaster::compare_geom(SpatRaster x, bool lyrs, bool crs, double tol, boo
 		}
 	}
 	if (res) {
-		if (! ((is_equal_relative(x.xres(), xres(), 0.0001)) & (is_equal_relative(x.yres(), yres(), 0.0001)))) {
+		if (! ((is_equal_relative(x.xres(), xres(), 0.0001)) && (is_equal_relative(x.yres(), yres(), 0.0001)))) {
 			setError("resolution does not match");
 			return false;
 		}
