@@ -105,6 +105,7 @@ class SpatOptions {
 		std::string tempdir = "";
 		bool todisk = false;
 		double memmax = -1;
+		double memmin = 134217728; // 1024^3 / 8
 		double memfrac = 0.6;
 		double tolerance = 0.1;
 		
@@ -148,6 +149,8 @@ class SpatOptions {
 		void set_memfrac(double d);
 		double get_memmax();
 		void set_memmax(double d);
+		double get_memmin();
+		void set_memmin(double d);
 		std::string get_tempdir();
 		void set_tempdir(std::string d);
 		double get_tolerance();
@@ -208,14 +211,16 @@ class SpatExtent {
 		SpatExtent(double _xmin, double _xmax, double _ymin, double _ymax) {xmin = _xmin; xmax = _xmax; ymin = _ymin; ymax = _ymax;}
 		virtual ~SpatExtent(){}
 
-
+		SpatExtent deepCopy() {return *this;}
 		SpatExtent align(double d, std::string snap);
 
-		void intersect(SpatExtent e) { // check first if intersects?
-			xmin = std::max(xmin, e.xmin);
-			xmax = std::min(xmax, e.xmax);
-			ymin = std::max(ymin, e.ymin);
-			ymax = std::min(ymax, e.ymax);
+		SpatExtent intersect(SpatExtent e) { // check first if intersects?
+			SpatExtent out;
+			out.xmin = std::max(xmin, e.xmin);
+			out.xmax = std::min(xmax, e.xmax);
+			out.ymin = std::max(ymin, e.ymin);
+			out.ymax = std::min(ymax, e.ymax);
+			return out;
 		}
 
 		void unite(SpatExtent e) {
