@@ -33,6 +33,8 @@ SpatDataFrame SpatDataFrame::skeleton() {
 	out.dv = std::vector<std::vector<double>>(dv.size());
 	out.iv = std::vector<std::vector<long>>(iv.size());
 	out.sv = std::vector<std::vector<std::string>>(sv.size());
+	out.bv = std::vector<std::vector<int8_t>>(bv.size());
+	out.tv = std::vector<std::vector<SpatTime_t>>(tv.size());
 	return out;
 }
 
@@ -64,12 +66,31 @@ std::vector<std::string> SpatDataFrame::getS(unsigned i) {
 	return sv[j];
 }
 
-
 std::string SpatDataFrame::getSvalue(unsigned i, unsigned j) {
 	j = iplace[j];
 	return sv[j][i];
 }
 
+std::vector<int8_t> SpatDataFrame::getB(unsigned i) {
+	unsigned j = iplace[i];
+	return bv[j];
+}
+
+int8_t SpatDataFrame::getBvalue(unsigned i, unsigned j) {
+	j = iplace[j];
+	return bv[j][i];
+}
+
+std::vector<SpatTime_t> SpatDataFrame::getT(unsigned i) {
+	unsigned j = iplace[i];
+	return tv[j];
+}
+
+SpatTime_t SpatDataFrame::getTvalue(unsigned i, unsigned j) {
+	j = iplace[j];
+	return tv[j][i];
+}
+
 
 SpatDataFrame SpatDataFrame::subset_rows(unsigned i) {
 	std::vector<unsigned> r = { i }; 
@@ -95,11 +116,10 @@ SpatDataFrame SpatDataFrame::subset_rows(std::vector<unsigned> range) {
 	out.dv.resize(dv.size());
 	out.iv.resize(iv.size());
 	out.sv.resize(sv.size());
-
-	out.dv.reserve(range.size());
-	out.iv.reserve(range.size());
-	out.sv.reserve(range.size());
-
+	out.bv.resize(bv.size());
+	out.tv.resize(tv.size());
+	out.reserve(range.size());
+	
 	for (size_t i=0; i < range.size(); i++) {
 		for (size_t j=0; j < dv.size(); j++) {
 			out.dv[j].push_back(dv[j][range[i]]);
@@ -110,6 +130,12 @@ SpatDataFrame SpatDataFrame::subset_rows(std::vector<unsigned> range) {
 		for (size_t j=0; j < sv.size(); j++) {
 			out.sv[j].push_back(sv[j][range[i]]);
 		}
+		for (size_t j=0; j < bv.size(); j++) {
+			out.bv[j].push_back(bv[j][range[i]]);
+		}
+		for (size_t j=0; j < tv.size(); j++) {
+			out.tv[j].push_back(tv[j][range[i]]);
+		}
 	}
 	return out;
 }
@@ -131,6 +157,8 @@ SpatDataFrame SpatDataFrame::subset_cols(std::vector<unsigned> range) {
 	unsigned dcnt=0;
 	unsigned icnt=0;
 	unsigned scnt=0;
+	unsigned bcnt=0;
+	unsigned tcnt=0;
 	for (size_t i=0; i < range.size(); i++) {
 		if (range[i] < 0 || range[i] >= ncol()) {
 			out.setError("invalid column");
@@ -149,11 +177,21 @@ SpatDataFrame SpatDataFrame::subset_cols(std::vector<unsigned> range) {
 			out.iplace.push_back(icnt);
 			out.itype.push_back(1);
 			icnt++;
-		} else {
+		} else if (itype[j] == 2) {
 			out.sv.push_back(sv[p]);
 			out.iplace.push_back(scnt);
 			out.itype.push_back(2);
 			scnt++;
+		} else if (itype[j] == 3) {
+			out.bv.push_back(bv[p]);
+			out.iplace.push_back(bcnt);
+			out.itype.push_back(3);
+			bcnt++;
+		} else {
+			out.tv.push_back(tv[p]);
+			out.iplace.push_back(tcnt);
+			out.itype.push_back(4);
+			tcnt++;
 		}
 	}
 	return out;
@@ -174,8 +212,12 @@ unsigned SpatDataFrame::nrow() {
 			n = dv[0].size();
 		} else if (itype[0] == 1) {
 			n = iv[0].size();
-		} else {
+		} else if (itype[0] == 2) {
 			n = sv[0].size();
+		} else if (itype[0] == 3) {
+			n = bv[0].size();
+		} else {
+			n = tv[0].size();
 		}
 	}
 	return n;
@@ -195,6 +237,13 @@ void SpatDataFrame::add_row() {
 	for (size_t i=0; i < sv.size(); i++) {
 		sv[i].push_back(NAS);
 	}
+	for (size_t i=0; i < bv.size(); i++) {
+		bv[i].push_back(2);
+	}
+	SpatTime_t timeNA = NA<SpatTime_t>::value;
+	for (size_t i=0; i < tv.size(); i++) {
+		tv[i].push_back(timeNA);
+	}
 }
 
 
@@ -210,6 +259,13 @@ void SpatDataFrame::add_rows(size_t n) {
 	for (size_t i=0; i < sv.size(); i++) {
 		sv[i].resize(s, NAS);
 	}
+	for (size_t i=0; i < bv.size(); i++) {
+		bv[i].resize(s, 2);
+	}
+	SpatTime_t timeNA = NA<SpatTime_t>::value;
+	for (size_t i=0; i < tv.size(); i++) {
+		tv[i].resize(s, timeNA);
+	}
 }
 
 
@@ -223,6 +279,12 @@ void SpatDataFrame::reserve(unsigned n) {
 	for (size_t i=0; i<sv.size(); i++) {
 		sv[i].reserve(n);
 	}
+	for (size_t i=0; i<bv.size(); i++) {
+		bv[i].reserve(n);
+	}
+	for (size_t i=0; i<tv.size(); i++) {
+		tv[i].reserve(n);
+	}
 }
 
 void SpatDataFrame::resize_rows(unsigned n) {
@@ -236,6 +298,13 @@ void SpatDataFrame::resize_rows(unsigned n) {
 	for (size_t i=0; i<sv.size(); i++) {
 		sv[i].resize(n, NAS);
 	}
+	for (size_t i=0; i<bv.size(); i++) {
+		bv[i].resize(n, 2);
+	}
+	SpatTime_t timeNA = NA<SpatTime_t>::value;
+	for (size_t i=0; i<tv.size(); i++) {
+		tv[i].resize(n, timeNA);
+	}
 }
 
 void SpatDataFrame::remove_rows(std::vector<unsigned> r) {
@@ -255,6 +324,12 @@ void SpatDataFrame::remove_rows(std::vector<unsigned> r) {
 		for (size_t i=0; i<sv.size(); i++) {
 			sv[i].erase(sv[i].begin() +r[j]);
 		}
+		for (size_t i=0; i<bv.size(); i++) {
+			bv[i].erase(bv[i].begin() +r[j]);
+		}
+		for (size_t i=0; i<tv.size(); i++) {
+			tv[i].erase(tv[i].begin() +r[j]);
+		}
 	}
 }
 
@@ -305,8 +380,12 @@ bool SpatDataFrame::remove_column(int i) {
 		dv.erase(dv.begin()+place);
 	} else if (dtype == 1) {
 		iv.erase(iv.begin()+place);
-	} else {
+	} else if (dtype == 2) {
 		sv.erase(sv.begin()+place);
+	} else if (dtype == 3) {
+		bv.erase(bv.begin()+place);
+	} else {
+		tv.erase(tv.begin()+place);
 	}
 	return true;
 }
@@ -344,6 +423,46 @@ bool SpatDataFrame::add_column(std::vector<std::string> x, std::string name) {
 	return true;
 }
 
+bool SpatDataFrame::add_column(std::vector<int8_t> x, std::string name) {
+	unsigned nr = nrow();
+	if ((nr != 0) & (nr != x.size())) return false; 
+	iplace.push_back(bv.size());
+	itype.push_back(3);
+	names.push_back(name);
+	bv.push_back(x);
+	return true;
+}
+
+bool SpatDataFrame::add_column_bool(std::vector<int> x, std::string name) {
+	unsigned nr = nrow();
+	if ((nr != 0) & (nr != x.size())) return false; 
+	iplace.push_back(bv.size());
+	itype.push_back(3);
+	names.push_back(name);
+	std::vector<int8_t> b;
+	b.reserve(x.size());
+	for (size_t i=0; i<x.size(); i++){
+		if ((x[i] == 0)|| (x[i]==1)) {
+			b.push_back(x[i]);
+		} else {
+			b.push_back(2);
+		}
+	}
+	bv.push_back(b);
+	return true;
+}
+
+
+bool SpatDataFrame::add_column(std::vector<SpatTime_t> x, std::string name) {
+	unsigned nr = nrow();
+	if ((nr != 0) & (nr != x.size())) return false; 
+	iplace.push_back(tv.size());
+	itype.push_back(4);
+	names.push_back(name);
+	tv.push_back(x);
+	return true;
+}
+
 
 void SpatDataFrame::add_column(unsigned dtype, std::string name) {
 	unsigned nr = nrow();
@@ -356,10 +475,19 @@ void SpatDataFrame::add_column(unsigned dtype, std::string name) {
 		std::vector<long> iins(nr, longNA);
 		iplace.push_back(iv.size());
 		iv.push_back(iins);
-	} else {
+	} else if (dtype == 2) {
 		std::vector<std::string> sins(nr, NAS);
 		iplace.push_back(sv.size());
 		sv.push_back(sins);
+	} else if (dtype == 3) {
+		std::vector<int8_t> bins(nr, 2);
+		iplace.push_back(bv.size());
+		bv.push_back(bins);
+	} else {
+		SpatTime_t timeNA = NA<SpatTime_t>::value;
+		std::vector<SpatTime_t> sins(nr, timeNA);
+		iplace.push_back(tv.size());
+		tv.push_back(sins);
 	}
 	itype.push_back(dtype);
 	names.push_back(name);
@@ -375,9 +503,15 @@ bool SpatDataFrame::cbind(SpatDataFrame &x) {
 		} else if (x.itype[i] == 1) {
 			std::vector<long> d = x.getI(i);
 			if (!add_column(d, nms[i])) return false;
-		} else {
+		} else if (x.itype[i] == 2) {
 			std::vector<std::string> d = x.getS(i);
 			if (!add_column(d, nms[i])) return false;
+		} else if (x.itype[i] == 3) {
+			std::vector<int8_t> d = x.getB(i);
+			if (!add_column(d, nms[i])) return false;
+		} else {
+			std::vector<SpatTime_t> d = x.getT(i);
+			if (!add_column(d, nms[i])) return false;
 		}
 	}
 	return true;
@@ -406,10 +540,18 @@ bool SpatDataFrame::rbind(SpatDataFrame &x) {
 				size_t a = iv.size()-1;
 				iv[a].insert(iv[a].begin()+nr1, 
 					x.iv[b].begin(), x.iv[b].end());
-			} else {
+			} else if (x.itype[i] == 2) { 
 				size_t a = sv.size()-1;
 				sv[a].insert(sv[a].begin()+nr1, 
 					x.sv[b].begin(), x.sv[b].end());
+			} else if (x.itype[i] == 3) { 
+				size_t a = bv.size()-1;
+				bv[a].insert(bv[a].begin()+nr1, 
+					x.bv[b].begin(), x.bv[b].end());
+			} else { 
+				size_t a = tv.size()-1;
+				tv[a].insert(tv[a].begin()+nr1, 
+					x.tv[b].begin(), x.tv[b].end());
 			} 
 		} else {
 			size_t a = iplace[j];
@@ -417,13 +559,19 @@ bool SpatDataFrame::rbind(SpatDataFrame &x) {
 			if (itype[j] == x.itype[i]) {
 				if (itype[j] == 0) { 
 					dv[a].insert(dv[a].begin()+nr1, 
-						x.dv[b].begin(), x.dv[b].end());
+					x.dv[b].begin(), x.dv[b].end());
 				} else if (itype[j] == 1) { 
 					iv[a].insert(iv[a].begin()+nr1, 
-						x.iv[b].begin(), x.iv[b].end());
-				} else {
+					x.iv[b].begin(), x.iv[b].end());
+				} else if (itype[j] == 2) { 
 					sv[a].insert(sv[a].begin()+nr1, 
-						x.sv[b].begin(), x.sv[b].end());
+					x.sv[b].begin(), x.sv[b].end());
+				} else if (itype[j] == 3) { 
+					bv[a].insert(bv[a].begin()+nr1, 
+					x.bv[b].begin(), x.bv[b].end());
+				} else { 
+					tv[a].insert(tv[a].begin()+nr1, 
+					x.tv[b].begin(), x.tv[b].end());
 				} 
 			} else {
 				if (itype[j] == 2) {
@@ -432,27 +580,86 @@ bool SpatDataFrame::rbind(SpatDataFrame &x) {
 						for (size_t k=0; k<nr2; k++) {
 							sv[a].push_back(std::to_string(x.dv[b][k]));
 						}
-					} else {
+					} else if (x.itype[i] == 1){
 						for (size_t k=0; k<nr2; k++) {
 							sv[a].push_back(std::to_string(x.iv[b][k]));
 						}
+					} else if (x.itype[i] == 3){
+						for (size_t k=0; k<nr2; k++) {
+							if (x.bv[b][k] == 0) {
+								sv[a].push_back("TRUE");
+							} else if (x.bv[b][k] == 1) {
+								sv[a].push_back("FALSE");
+							} else {
+								sv[a].push_back("NAS");								
+							}
+						}
+					} else {
+						for (size_t k=0; k<nr2; k++) {
+							sv[a].push_back(std::to_string(x.tv[b][k]));
+						}
 					}
 				} else if (itype[j] == 0) { 
+					dv[a].resize(nr1);
 					if (x.itype[i] == 1) {
-						dv[a].resize(nr1);
 						for (size_t k=0; k<nr2; k++) {
 							dv[a].push_back(x.iv[b][k]);
 						}
-					}
-				} else if (itype[j] == 1) { 
-					if (x.itype[i] == 0) {
-						iv[a].resize(nr1);
+					} else if (x.itype[i] == 2) {
+						// could try to_double instead
+						dv[a].resize(nr1 + nr2);
+					} else if (x.itype[i] == 3) {
 						for (size_t k=0; k<nr2; k++) {
-							iv[a].push_back(x.dv[b][k]);
+							if (x.iv[b][k] < 2) {
+								dv[a].push_back(x.bv[b][k]);
+							} else {
+								dv[a].push_back(NAN);								
+							}
+						}
+					} else {
+						for (size_t k=0; k<nr2; k++) {
+							dv[a].push_back(x.tv[b][k]);
 						}
 					}
+
+				} else if (itype[j] == 1) { 
+					iv[a].resize(nr1);
+					if (x.itype[i] == 0) {
+						long longNA = NA<long>::value;
+						for (size_t k=0; k<nr2; k++) {
+							if (std::isnan(x.dv[b][k])) {
+								iv[a].push_back(longNA);								
+							} else {
+								iv[a].push_back(x.dv[b][k]);
+							}
+						}
+					} else if (x.itype[i] == 3) {
+						long longNA = NA<long>::value;
+						for (size_t k=0; k<nr2; k++) {
+							if (x.bv[b][k] > 1) {
+								iv[a].push_back(longNA);								
+							} else {
+								iv[a].push_back(x.bv[b][k]);
+							}
+						}
+					} else {
+						iv[a].resize(nr1 + nr2);						
+					}
+				} else if (itype[j] == 3) {					
+					if (x.itype[i] == 1) {
+						for (size_t k=0; k<nr2; k++) {
+							if (x.dv[b][k] == 0 || x.dv[b][k] == 1) {
+								bv[a].push_back(x.dv[b][k]);								
+							} else {
+								bv[a].push_back(2);
+							}
+						}
+					} else {
+						bv[a].resize(nr1 + nr2);						
+					}
+				} else if (itype[j] == 4) {
+					tv[a].resize(nr1 + nr2);						
 				}
-				//degrade types
 			}
 		}
 	}
@@ -479,7 +686,7 @@ void SpatDataFrame::set_names(std::vector<std::string> nms){
 
 
 std::vector<std::string> SpatDataFrame::get_datatypes() {
-	std::vector<std::string> types = {"double", "long", "string"};
+	std::vector<std::string> types = {"double", "long", "string", "bool", "time"};
 	std::vector<std::string> stype(itype.size());
 	for (size_t i=0; i<itype.size(); i++) {
 		stype[i] = types[itype[i]]; 
@@ -492,13 +699,13 @@ std::string SpatDataFrame::get_datatype(std::string field) {
 	int i = where_in_vector(field, get_names(), false);
 	if (i < 0) return "";
 	i = itype[i]; 
-	std::vector<std::string> types = {"double", "long", "string"};
+	std::vector<std::string> types = {"double", "long", "string", "bool", "time"};
 	return types[i]; 
 }
 
 std::string SpatDataFrame::get_datatype(int field) {
 	if ((field < 0) || (field > (int)(ncol()-1))) return "";
-	std::vector<std::string> types = {"double", "long", "string"};
+	std::vector<std::string> types = {"double", "long", "string", "bool", "time"};
 	return types[itype[field]]; 
 }
 
@@ -529,9 +736,15 @@ SpatDataFrame SpatDataFrame::unique(int col) {
 	} else if (out.itype[0] == 1) {
 		std::sort(out.iv[0].begin(), out.iv[0].end());
 		out.iv[0].erase(std::unique(out.iv[0].begin(), out.iv[0].end()), out.iv[0].end());
-	} else {
+	} else if (out.itype[0] == 2) {
 		std::sort(out.sv[0].begin(), out.sv[0].end());
 		out.sv[0].erase(std::unique(out.sv[0].begin(), out.sv[0].end()), out.sv[0].end());
+	} else if (out.itype[0] == 3) {
+		std::sort(out.bv[0].begin(), out.bv[0].end());
+		out.bv[0].erase(std::unique(out.bv[0].begin(), out.bv[0].end()), out.bv[0].end());
+	} else if (out.itype[0] == 4) {
+		std::sort(out.tv[0].begin(), out.tv[0].end());
+		out.tv[0].erase(std::unique(out.tv[0].begin(), out.tv[0].end()), out.tv[0].end());
 	}
 	return out;
 }
@@ -564,7 +777,7 @@ std::vector<int> SpatDataFrame::getIndex(int col, SpatDataFrame &x) {
 				}
 			}
 		}
-	} else {
+	} else if (x.itype[0] == 2) {
 		for (size_t i=0; i<nd; i++) {
 			for (size_t j=0; j<nu; j++) {
 				if (sv[ccol][i] == x.sv[0][j]) {
@@ -573,7 +786,139 @@ std::vector<int> SpatDataFrame::getIndex(int col, SpatDataFrame &x) {
 				}
 			}
 		}
+	} else if (x.itype[0] == 3) {
+		for (size_t i=0; i<nd; i++) {
+			for (size_t j=0; j<nu; j++) {
+				if (bv[ccol][i] == x.bv[0][j]) {
+					idx[i] = j;
+					continue;
+				}
+			}
+		}
+	} else if (x.itype[0] == 4) {
+		for (size_t i=0; i<nd; i++) {
+			for (size_t j=0; j<nu; j++) {
+				if (tv[ccol][i] == x.tv[0][j]) {
+					idx[i] = j;
+					continue;
+				}
+			}
+		}
 	}
 	return idx;
 }
 
+
+std::vector<double> SpatDataFrame::as_double(size_t v) {
+	std::vector<double> out;
+	if (v >= ncol()) {
+		setError("attempting to read a column that does not exist");
+		return out;
+	}
+	if (itype[v] == 2) {
+		setError("as_double not available for string");
+		return out;
+	}
+	size_t j = iplace[v];
+	if (itype[v] == 0) return dv[j];
+//	if (itype[v] == 1) {
+	out.reserve(nrow());
+	if (itype[v]==1) {
+		long longNA = NA<long>::value;		
+		for (size_t i=0; i<nrow(); i++){
+			if (tv[j][i] == longNA) {
+				out[i] = NAN;	
+			} else {
+				out[i] = (double)iv[j][i];			
+			}
+		}	
+	} else if (itype[v]==3) {
+		for (size_t i=0; i<nrow(); i++){
+			if (bv[j][i] > 1) {
+				out[i] = NAN;
+			} else {
+				out[i] = (double)bv[j][i];			
+			}
+		}
+	} else if (itype[v]==4) {
+		SpatTime_t timeNA = NA<SpatTime_t>::value;
+		for (size_t i=0; i<nrow(); i++){
+			if (tv[j][i] == timeNA) {
+				out[i] = NAN;				
+			} else {
+				out[i] = (double)tv[j][i];
+			}
+		}
+	} 
+	return out;
+}	
+	
+
+std::vector<long> SpatDataFrame::as_long(size_t v) {
+	std::vector<long> out;
+	if (v >= ncol()) {
+		setError("attempting to read a column that does not exist");
+		return out;
+	}
+	if (itype[v] == 2) {
+		setError("as_long not available for string");
+		return out;
+	}
+	size_t j = iplace[v];
+	if (itype[v] == 1) return iv[j];
+//	if (itype[v] == 0) {
+	out.reserve(nrow());
+	long longNA = NA<long>::value;		
+	if (itype[v] == 0) {
+		for (size_t i=0; i<nrow(); i++){
+			if (std::isnan(dv[j][i])) {
+				out.push_back(longNA);			
+			} else {
+				out.push_back( (long)dv[j][i] );
+			}
+		}
+	} else if (itype[v]==3) {
+		for (size_t i=0; i<nrow(); i++){
+			if (bv[j][i] > 1) {
+				out[i] = longNA;
+			} else {
+				out[i] = (long) bv[j][i];			
+			}
+		}
+	} else if (itype[v]==4) {
+		SpatTime_t timeNA = NA<SpatTime_t>::value;
+		for (size_t i=0; i<nrow(); i++){
+			if (tv[j][i] == timeNA) {
+				out[i] = longNA;				
+			} else {
+				out[i] = (long) tv[j][i];
+			}
+		}
+	} 
+	return out;
+}	
+	
+
+std::vector<std::string> SpatDataFrame::as_string(size_t v) {
+	std::vector<std::string> out;
+	if (v >= ncol()) {
+		setError("attempting to read a column that does not exist");
+		return out;
+	}
+	std::string dt = get_datatype(v);
+	size_t j = iplace[v];
+
+	if (dt == "string") return sv[j];
+	out.reserve(nrow());
+	if (dt == "double") {
+		for (size_t i=0; i<nrow(); i++) {
+			out.push_back(double_to_string(dv[j][i]));
+		}
+	} else if (dt == "long") {
+		for (size_t i=0; i<nrow(); i++) {
+			out.push_back(std::to_string(iv[j][i]));
+		}
+	}
+	return out;
+}
+
