@@ -20,6 +20,7 @@
 #include <string>
 //#include "spatMessages.h"
 #include "spatBase.h"
+#include "spatTime.h"
 
 class SpatDataFrame {
 	public:
@@ -37,11 +38,13 @@ class SpatDataFrame {
 		std::string getError() { return msg.getError(); }
 	
 		std::vector<std::string> names;
-		std::vector<unsigned> itype; //0 double, 1 long, 2 string
+		std::vector<unsigned> itype; //0 double, 1 long, 2 string, 3 bool, 4 time
 		std::vector<unsigned> iplace;
 		std::vector< std::vector<double>> dv;
 		std::vector< std::vector<long>> iv;
 		std::vector< std::vector<std::string>> sv;
+		std::vector< std::vector<int8_t>> bv;
+		std::vector< std::vector<SpatTime_t>> tv;
 		std::string NAS = "____NA_+";
 		
 		unsigned nrow();
@@ -54,10 +57,18 @@ class SpatDataFrame {
 		std::vector<double> getD(unsigned i);
 		std::vector<long> getI(unsigned i);
 		std::vector<std::string> getS(unsigned i);
+		std::vector<int8_t> getB(unsigned i);
+		std::vector<SpatTime_t> getT(unsigned i);
+
+		std::vector<std::string> as_string(size_t v);
+		std::vector<long> as_long(size_t v);
+		std::vector<double> as_double(size_t v);
 
 		double getDvalue(unsigned i, unsigned j);
 		long getIvalue(unsigned i, unsigned j);
 		std::string getSvalue(unsigned i, unsigned j);
+		int8_t getBvalue(unsigned i, unsigned j);
+		SpatTime_t getTvalue(unsigned i, unsigned j);
 	
 		void add_row();
 		void add_rows(size_t n);
@@ -71,10 +82,15 @@ class SpatDataFrame {
 		bool add_column(std::vector<long> x, std::string name);
 		bool add_column(std::vector<int> x, std::string name);
 		bool add_column(std::vector<std::string> x, std::string name);
+		bool add_column(std::vector<int8_t> x, std::string name);
+		bool add_column(std::vector<SpatTime_t> x, std::string name);
+		bool add_column_bool(std::vector<int> x, std::string name);
 		
 		void insert_column(std::vector<double>, size_t i);
 		void insert_column(std::vector<long>, size_t i);		
 		void insert_column(std::vector<std::string>, size_t i);
+		void insert_column(std::vector<int8_t>, size_t i);
+		void insert_column(std::vector<SpatTime_t>, size_t i);
 
 		bool remove_column(std::string field);
 		bool remove_column(int i);		
