@@ -49,6 +49,7 @@ SpatRaster::SpatRaster(std::vector<std::string> fname, std::vector<int> subds, s
 		setError("cannot open file: " + fname[0]);
 		return;
 	}
+	SpatOptions opt;
 	for (size_t i=1; i<fname.size(); i++) {
 		SpatRaster r;
 		bool ok = r.constructFromFile(fname[i], subds, subdsname, options);
@@ -57,7 +58,7 @@ SpatRaster::SpatRaster(std::vector<std::string> fname, std::vector<int> subds, s
 		}
 
 		if (ok) {
-			addSource(r, false);
+			addSource(r, false, opt);
 			if (r.msg.has_error) {
 				setError(r.msg.error);
 				return;
@@ -297,8 +298,9 @@ SpatRaster SpatRaster::setResolution(double xres, double yres) {
 		return(out);
 	}
 	SpatExtent e = getExtent();
-	unsigned nc = round((e.xmax-e.xmin) / xres);
-	unsigned nr = round((e.ymax-e.ymin) / yres);
+	unsigned nc = std::max(1., round((e.xmax-e.xmin) / xres));
+	unsigned nr = std::max(1., round((e.ymax-e.ymin) / yres));
+
 	double xmax = e.xmin + nc * xres;
 	double ymax = e.ymin + nr * yres;
 	unsigned nl = nlyr();
@@ -409,6 +411,22 @@ bool SpatRaster::is_global_lonlat() {
 	return source[0].srs.is_global_lonlat(e);
 }
 
+int SpatRaster::ns_polar() {
+	int polar = 0;
+	if (!is_lonlat()) {
+		return polar;
+	} 
+	SpatExtent e = getExtent();
+	if ((e.ymax - 90) > -0.00001) {
+		polar = 1;
+	}
+	if ((e.ymin + 90) < 0.00001) {
+		polar = polar == 1 ? 2 : -1;
+	}
+	return polar;
+}
+
+
 
 bool SpatRaster::sources_from_file() {
 	for (size_t i=0; i<source.size(); i++) {
@@ -430,7 +448,7 @@ SpatRaster SpatRaster::sources_to_disk(std::vector<std::string> &tmpfs, bool uni
 
 	std::string tmpbasename = tempFile(opt.get_tempdir(), opt.pid, "_temp_");
 
-
+	SpatOptions ops(opt);
 	for (size_t i=0; i<nsrc; i++) {
 		bool write = false;
 		if (!source[i].in_order() || source[i].memory) {
@@ -453,7 +471,7 @@ SpatRaster SpatRaster::sources_to_disk(std::vector<std::string> &tmpfs, bool uni
 		if (i == 0) {
 			out.setSource(rs.source[0]);
 		} else {
-			out.addSource(rs, false);
+			out.addSource(rs, false, ops);
 		}
 	}
 	return out;
@@ -774,9 +792,13 @@ bool SpatRaster::setDepth(std::vector<double> depths) {
 
 bool SpatRaster::setUnit(std::vector<std::string> units) {
 	if (units.size() == 1) {
+		bool hu = true;
+		if (units[0] == "") {
+			hu = false;
+		}	
         for (size_t i=0; i<source.size(); i++)	{
             source[i].unit = std::vector<std::string> (source[i].nlyr, units[0]);
-			source[i].hasUnit = true;
+			source[i].hasUnit = hu;
         }
         return true;
 	} else if (units.size() != nlyr()) {
@@ -975,21 +997,21 @@ SpatRaster SpatRaster::replace(SpatRaster x, unsigned layer, SpatOptions &opt) {
 		lyrs.resize(n-1);
 		std::iota(lyrs.begin(), lyrs.end(), 1);
 		SpatRaster r = subset(lyrs, fopt);
-		out.addSource(r, false);
+		out.addSource(r, false, fopt);
 	} else if (layer == n-1) {
 		lyrs.resize(n-1);
 		std::iota(lyrs.begin(), lyrs.end(), 0);
 		out = subset(lyrs, fopt);
-		out.addSource(x, false);
+		out.addSource(x, false, fopt);
 	} else {
 		lyrs.resize(layer);
 		std::iota(lyrs.begin(), lyrs.end(), 0);
 		out = subset(lyrs, fopt);
-		out.addSource(x, false);
+		out.addSource(x, false, fopt);
 		lyrs.resize(n-layer-1);
 		std::iota(lyrs.begin(), lyrs.end(), layer+1);
 		SpatRaster r = subset(lyrs, fopt);
-		out.addSource(r, false);
+		out.addSource(r, false, fopt);
 	}
 	return out;
 }
@@ -1009,7 +1031,7 @@ SpatRaster SpatRaster::makeCategorical(unsigned layer, SpatOptions &opt) {
 
 	r.math2("round", 0, fopt);
 
-	std::vector<std::vector<double>> u = r.unique(false, fopt);
+	std::vector<std::vector<double>> u = r.unique(false, true, fopt);
 /*
 	std::vector<double> id(u[0].size());
 	std::iota(id.begin(), id.end(), 0);
@@ -1049,7 +1071,7 @@ bool SpatRaster::createCategories(unsigned layer, SpatOptions &opt) {
 	}
 	std::vector<unsigned> lyrs(1, layer);
 	SpatRaster r = subset(lyrs, opt);
-	std::vector<std::vector<double>> u = r.unique(false, opt);
+	std::vector<std::vector<double>> u = r.unique(false, true, opt);
     std::vector<unsigned> sl = findLyr(layer);
 
 	std::vector<std::string> s(u[0].size());
@@ -1342,24 +1364,21 @@ double SpatRaster::cellFromRowCol (int_64 row, int_64 col) {
 }
 
 std::vector<double> SpatRaster::cellFromRowColCombine(std::vector<int_64> row, std::vector<int_64> col) {
-	recycle(row, col);
 	size_t n = row.size();
-	int_64 nc = ncol();
-	int_64 nr = nrow();
-
-	std::vector<double> x(n * n);
+	size_t m = col.size();
+	double nc = ncol();
+	double nr = nrow();
+	std::vector<double> x;
+	x.reserve(n * m);
 	for (size_t i=0; i<n; i++) {
-		for (size_t j=0; j<n; j++) {
-			x[i*n+j] = (row[i]<0 || row[i] >= nr || col[j]<0 || col[j] >= nc) ? NAN : row[i] * nc + col[j];
+		for (size_t j=0; j<m; j++) {
+			if (row[i] < 0 || row[i] >= nr || col[j]<0 || col[j] >= nc) {
+				x.push_back(NAN);				
+			} else {
+				x.push_back(row[i] * nc + col[j]);
+			}
 		}
 	}
-	// duplicates occur if recycling occurs
-	// could be avoided by smarter combination
-	x.erase(std::remove_if(x.begin(), x.end(),
-            [](const double& value) { return std::isnan(value); }), x.end());
-
-	std::sort(x.begin(), x.end());
-	x.erase(std::unique(x.begin(), x.end()), x.end());
 	return x;
 }
 
@@ -1657,7 +1676,7 @@ std::vector<double> SpatRaster::adjacent(std::vector<double> cells, std::string
             rows = {r[i]-2, r[i]-2, r[i]-1, r[i]-1, r[i]-1, r[i]-1, r[i]-1, r[i]  , r[i]  , r[i]+1, r[i]+1, r[i]+1, r[i]+1, r[i]+1, r[i]+2, r[i]+2};
             cols = {c[i]-1, c[i]+1, c[i]-2, c[i]-1, c[i],   c[i]+1, c[i]+2, c[i]-1, c[i]+1, c[i]-2, c[i]-1, c[i]  , c[i]+1, c[i]+2, c[i]-1, c[i]+1};
             if (globlatlon) {
-                if ((c[i]==0) | (c[i]==1)) {
+                if ((c[i]==0) || (c[i]==1)) {
                     for (size_t j=0; j<16; j++) {
                         cols[j] = (cols[j] < 0) ? nc-cols[j] : cols[j];
                     }
@@ -1678,7 +1697,7 @@ std::vector<double> SpatRaster::adjacent(std::vector<double> cells, std::string
 }
 
 
-SpatVector SpatRaster::as_points(bool values, bool narm, SpatOptions &opt) {
+SpatVector SpatRaster::as_points(bool values, bool narm, bool nall, SpatOptions &opt) {
 
 	BlockSize bs = getBlockSize(opt);
 	std::vector<double> v, vout;
@@ -1719,15 +1738,29 @@ SpatVector SpatRaster::as_points(bool values, bool narm, SpatOptions &opt) {
  		size_t vnc = bs.nrows[i] * nc;
 		if (narm) {
 			for (size_t j=0; j<vnc; j++) {
-				bool foundna = false;
-				for (size_t lyr=0; lyr<nl; lyr++) {
-                    size_t off2 = lyr*vnc;
-                    if (std::isnan(v[off2+j])) {
-                        foundna = true;
-                        continue;
-                    }
-                }
-                if (foundna) continue;
+				if (nall) {
+					bool allna = true;
+					for (size_t lyr=0; lyr<nl; lyr++) {
+						size_t off2 = lyr*vnc;
+						if (!std::isnan(v[off2+j])) {
+							allna = false;
+							continue;
+						}
+					}
+					if (allna) continue;				
+				} else {
+					bool foundna = false;
+					for (size_t lyr=0; lyr<nl; lyr++) {
+						size_t off2 = lyr*vnc;
+						if (std::isnan(v[off2+j])) {
+							foundna = true;
+							continue;
+						}
+					}
+					if (foundna) continue;
+				}
+ 			
+				
                 xy = xyFromCell( off1+j );
                 SpatPart p(xy[0], xy[1]);
                 g.addPart(p);
@@ -1834,7 +1867,7 @@ SpatVector SpatRaster::as_polygons(bool values, bool narm) {
 
 */
 
-SpatVector SpatRaster::as_polygons(bool trunc, bool dissolve, bool values, bool narm, SpatOptions &opt) {
+SpatVector SpatRaster::as_polygons(bool trunc, bool dissolve, bool values, bool narm, bool nall, SpatOptions &opt) {
 
 	if (!hasValues()) {
 		values = false;
@@ -1886,11 +1919,22 @@ SpatVector SpatRaster::as_polygons(bool trunc, bool dissolve, bool values, bool
 	std::vector< std::vector<double> > xy = xyFromCell(cells);
 	for (int i=nc-1; i>=0; i--) {
 		if (narm) {
-			bool erase = false;
-			for (size_t j=0; j<nl; j++) {
-				if (std::isnan(vect.df.dv[j][i])) {
-					erase=true;
-					break;
+			bool erase;
+			if (nall) {
+				erase = true;
+				for (size_t j=0; j<nl; j++) {
+					if (!std::isnan(vect.df.dv[j][i])) {
+						erase=false;
+						break;
+					}
+				}
+			} else {
+				erase = false;
+				for (size_t j=0; j<nl; j++) {
+					if (std::isnan(vect.df.dv[j][i])) {
+						erase=true;
+						break;
+					}
 				}
 			}
 			if (erase) {
@@ -2010,7 +2054,7 @@ void SpatRaster::removeRGB(){
 
 
 bool SpatRaster::to_memory(SpatOptions &opt) {
-	if ((nsrc() == 1) & (source[0].memory)) {
+	if ((nsrc() == 1) && (source[0].memory)) {
 		return true;
 	}
 	SpatRaster g = geometry();
