@@ -310,6 +310,8 @@ std::string SpatVector::type(){
 		return "lines";
 	} else if (geoms[0].gtype == polygons) {
 		return "polygons";
+	} else if (geoms[0].gtype == null) {
+		return("null");		
 	} else {
 		return("unknown");
 	}
@@ -347,6 +349,17 @@ void SpatVector::computeExtent() {
 	}
 }
 
+std::vector<unsigned> SpatVector::nullGeoms(){
+	std::vector<unsigned> ids;
+	for (size_t i=0; i<geoms.size(); i++) {
+		if ((geoms[i].gtype == null) || (geoms[i].gtype == unknown)) {
+			ids.push_back(i);
+		}
+	}
+	return ids;
+}
+
+
 
 bool SpatVector::replaceGeom(SpatGeom p, unsigned i) {
 	if (i < geoms.size()) {
@@ -412,6 +425,7 @@ std::vector<std::vector<double>> SpatVector::coordinates() {
 }
 */
 
+
 size_t SpatVector::ncoords() {
 	size_t ncrds = 0;
 	size_t ng = geoms.size();
@@ -426,9 +440,10 @@ size_t SpatVector::ncoords() {
 			}
 		}
 	}
-	return ng;
+	return ncrds;
 }
 
+
 std::vector<std::vector<double>> SpatVector::coordinates() {
 	std::vector<std::vector<double>> out(2);
 	size_t ncrds = ncoords();
@@ -436,13 +451,10 @@ std::vector<std::vector<double>> SpatVector::coordinates() {
 	out[1].reserve(ncrds);
 	size_t ng = size();
 	for (size_t i=0; i<ng; i++) {
-		size_t np = geoms[i].size();
+		size_t np = geoms[i].size();		
 		for (size_t j=0; j<np; j++) {
-			size_t nx = geoms[i].parts[j].x.size();
-			for (size_t q=0; q < nx; q++) {
-				out[0].insert(out[0].end(), geoms[i].parts[j].x.begin(), geoms[i].parts[j].x.end());
-				out[1].insert(out[1].end(), geoms[i].parts[j].y.begin(), geoms[i].parts[j].y.end());
-			}
+			out[0].insert(out[0].end(), geoms[i].parts[j].x.begin(), geoms[i].parts[j].x.end());
+			out[1].insert(out[1].end(), geoms[i].parts[j].y.begin(), geoms[i].parts[j].y.end());
 			if (geoms[i].parts[j].hasHoles()) {
 				size_t nh = geoms[i].parts[j].nHoles();
 				for (size_t k=0; k < nh; k++) {
@@ -767,14 +779,27 @@ void SpatVector::setPointsGeometry(std::vector<double> &x, std::vector<double> &
 
 
 
-void SpatVector::setPointsDF(SpatDataFrame &x, std::vector<unsigned> geo, std::string crs) {
+void SpatVector::setPointsDF(SpatDataFrame &x, std::vector<unsigned> geo, std::string crs, bool keepgeom) {
 	if (x.nrow() == 0) return;
 	if ((x.itype[geo[0]] != 0) || (x.itype[geo[1]] != 0)) {
 		setError("coordinates must be numeric");
 		return;
 	}
+	if (geo[0] == geo[1]) {
+		setError("x and y coordinates are the same variable");
+		return;			
+	}
 	setPointsGeometry(x.dv[x.iplace[geo[0]]], x.dv[x.iplace[geo[1]]]);
-	setSRS( {crs});
+	setSRS( {crs} );
+	if (!keepgeom) {
+		if (geo[0] > geo[1]) {
+			x.remove_column(geo[0]);
+			x.remove_column(geo[1]);
+		} else {
+			x.remove_column(geo[1]);
+			x.remove_column(geo[0]);
+		}
+	}
 	df = x;
 }
 
@@ -785,7 +810,7 @@ SpatVector SpatVector::subset_rows(std::vector<int> range) {
 	int n = nrow();
 	std::vector<unsigned> r;
 	for (size_t i=0; i<range.size(); i++) {
-		if ((range[i] >= 0) & (range[i] < n)) {
+		if ((range[i] >= 0) && (range[i] < n)) {
 			r.push_back(range[i]);
 		}
 	}
@@ -853,7 +878,7 @@ SpatVector SpatVector::subset_cols(std::vector<int> range) {
 	std::vector<unsigned> valid;
 	valid.reserve(range.size());
 	for (size_t i=0; i<range.size(); i++) {
-		if ((range[i] >= 0) & (range[i] < nc)) {
+		if ((range[i] >= 0) && (range[i] < nc)) {
 			valid.push_back(range[i]);
 		}
 	}
