@@ -24,7 +24,7 @@
 #endif
 
 
-enum SpatGeomType { points, lines, polygons, unknown };
+enum SpatGeomType { points, lines, polygons, unknown, null };
 
 
 class SpatHole {
@@ -104,7 +104,13 @@ class SpatVector {
 		SpatDataFrame df;
 		//std::vector<std::string> crs;
 		SpatSRS srs;
-
+		bool is_proxy = false;
+		std::string read_query = "";
+		std::vector<double> read_extent;
+		std::string source = "";
+		std::string source_layer = "";
+		size_t geom_count = 0;
+		
 		SpatVector();
 		//SpatVector(const SpatVector &x);
 		SpatVector(SpatGeom g);
@@ -172,7 +178,7 @@ class SpatVector {
 
 		void setGeometry(std::string type, std::vector<unsigned> gid, std::vector<unsigned> part, std::vector<double> x, std::vector<double> y, std::vector<unsigned> hole);
 		void setPointsGeometry(std::vector<double> &x, std::vector<double> &y);
-		void setPointsDF(SpatDataFrame &x, std::vector<unsigned> geo, std::string crs);
+		void setPointsDF(SpatDataFrame &x, std::vector<unsigned> geo, std::string crs, bool keepgeom);
 
 		std::vector<double> area(std::string unit, bool transform, std::vector<double> mask);
 
@@ -190,17 +196,19 @@ class SpatVector {
 		SpatVector get_holes();
 		SpatVector set_holes(SpatVector x, size_t i);
 		SpatVector remove_duplicate_nodes(int digits);
-
-		bool read(std::string fname, std::string layer, std::string query, std::vector<double> extent, SpatVector filter);
 		
-		bool write(std::string filename, std::string lyrname, std::string driver, bool overwrite, std::vector<std::string>);
+		bool read(std::string fname, std::string layer, std::string query, std::vector<double> extent, SpatVector filter, bool as_proxy);
+		
+		bool write(std::string filename, std::string lyrname, std::string driver, bool append, bool overwrite, std::vector<std::string>);
 		
 #ifdef useGDAL
-		GDALDataset* write_ogr(std::string filename, std::string lyrname, std::string driver, bool overwrite, std::vector<std::string> options);
+		GDALDataset* write_ogr(std::string filename, std::string lyrname, std::string driver, bool append, bool overwrite, std::vector<std::string> options);
 		GDALDataset* GDAL_ds();
-		bool read_ogr(GDALDataset *poDS, std::string layer, std::string query, std::vector<double> extent, SpatVector filter);
+		bool read_ogr(GDALDataset *poDS, std::string layer, std::string query, std::vector<double> extent, SpatVector filter, bool as_proxy);
 		SpatVector fromDS(GDALDataset *poDS);
 		bool ogr_geoms(std::vector<OGRGeometryH> &ogrgeoms, std::string &message);		
+		bool delete_layers(std::string filename, std::vector<std::string> layers, bool return_error);		
+		std::vector<std::string> layer_names(std::string filename);		
 #endif
 
 // attributes
@@ -217,6 +225,9 @@ class SpatVector {
 		bool add_column(std::vector<T> x, std::string name) {
 			return df.add_column(x, name);
 		}
+		bool add_column_bool(std::vector<int> x, std::string name) {
+			return df.add_column_bool(x, name);
+		}
 
 		void remove_df() {
 			SpatDataFrame empty;
@@ -281,6 +292,8 @@ class SpatVector {
 		SpatVector point_buffer(std::vector<double>	 d, unsigned quadsegs, bool no_multipolygons);
 
 		SpatVector centroid(bool check_lonlat);
+		SpatVector point_on_surface(bool check_lonlat);
+
 		SpatVector crop(SpatExtent e);
 		SpatVector crop(SpatVector e);
 		SpatVector voronoi(SpatVector e, double tolerance, int onlyEdges);		
@@ -292,10 +305,12 @@ class SpatVector {
 		SpatVector erase_agg(SpatVector v);
 		SpatVector erase(SpatVector v);
 		SpatVector erase();
+		SpatVector mask(SpatVector x, bool inverse);
 		SpatVector gaps();		
 		SpatVector cover(SpatVector v, bool identity);
 		SpatVectorCollection split(std::string field);
 		SpatVector symdif(SpatVector v);
+		std::vector<bool> is_related(SpatVector v, std::string relation);
 		std::vector<int> relate(SpatVector v, std::string relation);
 		std::vector<int> relate(std::string relation, bool symmetrical);
 		std::vector<int> relateFirst(SpatVector v, std::string relation);
@@ -318,18 +333,20 @@ class SpatVector {
 		SpatVector cross_dateline(bool &fixed);
 		SpatVector densify(double interval, bool adjust);
 		SpatVector round(int digits);
+		std::vector<unsigned> nullGeoms();
 };
 
 
 
 class SpatVectorCollection {
 
-	private:
-		std::vector<SpatVector> v;
 
 	public:
 		virtual ~SpatVectorCollection(){}
-	
+		SpatVectorCollection deepCopy() { return *this; }
+
+		std::vector<SpatVector> v;
+
 		SpatMessages msg;
 		void setError(std::string s) { msg.setError(s); }
 		void addWarning(std::string s) { msg.addWarning(s); }
@@ -370,6 +387,18 @@ class SpatVectorCollection {
 		}
 		
 		SpatVector append();
+		SpatVectorCollection from_hex_col(std::vector<std::string> x, std::string srs);
 		
 };
 
+
+
+class SpatVectorProxy {
+	public:
+		SpatVector v;
+		SpatVectorProxy(){}
+		virtual ~SpatVectorProxy(){}
+		SpatVectorProxy deepCopy() {return *this;}
+		SpatVector query_filter(std::string query, std::vector<double> extent, SpatVector filter);
+};
+
