@@ -17,199 +17,77 @@
 
 #include "spatVector.h"
 #include "string_utils.h"
+#include <stdexcept>
 
 #ifdef useGDAL
 
 #include "file_utils.h"
 #include "ogrsf_frmts.h"
 
-
-/*
-bool SpatVector::ogr_geoms(std::vector<OGRGeometryH> &ogrgeoms, std::string &	message) {
-
-	const char *pszDriverName = "memory";
-	GDALDriverH hDriver;
-	hDriver = GDALGetDriverByName( pszDriverName );
-	GDALDatasetH hDS;
-	hDS = GDALCreate( hDriver, "point_out.shp", 0, 0, 0, GDT_Unknown, NULL );
-	if (hDS == NULL) {
-		return false;
-	}
-
-//    char **papszMetadata;
-
-	OGRwkbGeometryType wkb;
-	SpatGeomType geomtype = geoms[0].gtype;
-	if (geomtype == points) {
-		wkb = wkbPoint;
-	} else if (geomtype == lines) {
-		wkb = wkbMultiLineString;
-	} else if (geomtype == polygons) {
-		wkb = wkbMultiPolygon;
-	} else {
-        setError("this geometry type is not supported");
-        return false;
-	}
-
-	std::string s = srs.wkt;
-
-//	OGRSpatialReferenceH hSRS = OSRNewSpatialReference( NULL );
-//	OGRErr erro = OSRSetFromUserInput(hSRS, s.c_str());
-//	if (erro != 4) {
-//		return false ;
-//		char *pszSRS_WKT = NULL;
-//		OSRExportToWkt( hSRS, &pszSRS_WKT );
-//		OSRDestroySpatialReference( hSRS );
-//		GDALSetProjection( hDS, pszSRS_WKT );
-//		CPLFree( pszSRS_WKT );
-//	}
-
-
-	OGRSpatialReference *SRS = NULL;
-	if (s != "") {
-		SRS = new OGRSpatialReference;
-		OGRErr err = SRS->SetFromUserInput(s.c_str()); 
-		if (err != OGRERR_NONE) {
-			setError("crs error");
-			delete SRS;
-			return false;
-		}
-	}
-
-	OGRLayerH hLayer;
-	hLayer = GDALDatasetCreateLayer( hDS, "", SRS, wkb, NULL );
-    if( hLayer == NULL ) {
-        message = "Layer creation failed" ;
-        return false;
-    }
-
-	if (SRS != NULL) SRS->Release();
-
-	for (size_t i=0; i<ngeoms; i++) {
-
-	    OGRFeatureH hFeature;
-        OGRGeometryH hPt;
-
-        hFeature = OGR_F_Create( OGR_L_GetLayerDefn( hLayer ) );
-
-
-// points -- also need to do multipoints
-		OGRGeometryH hPt;
-		if (wkb == wkbPoint) {
-			SpatGeom g = getGeom(i);
-			OGR_G_SetPoint_2D(hPt, 0, g.parts[0].x[0], g.parts[0].y[0]);
-			ogrgeoms.push_back(hPt);
-// lines
-		} else if (wkb == wkbMultiLineString) {
-			SpatGeom g = getGeom(i);
-			OGRMultiLineString poGeom;
-			for (size_t j=0; j<g.size(); j++) {
-				OGRLineString poLine = OGRLineString();
-				SpatPart p = g.getPart(j);
-				for (size_t k=0; k<p.size(); k++) {
-					pt.setX(p.x[k]);
-					pt.setY(p.y[k]);
-					poLine.setPoint(k, &pt);
-				}
-				if (poGeom.addGeometry(&poLine) != OGRERR_NONE ) {
-					message = "cannot add line";
-					return false;
-				}
-			}
-			if (poFeature->SetGeometry( &poGeom ) != OGRERR_NONE) {
-				message = "cannot set geometry";
-				return false;
-			}
-
-// polygons
-		} else if (wkb == wkbMultiPolygon) {
-			SpatGeom g = getGeom(i);
-			OGRPolygon poGeom;
-			for (size_t j=0; j<g.size(); j++) {
-				OGRLinearRing poRing;
-				SpatPart p = g.getPart(j);
-				for (size_t k=0; k<p.size(); k++) {
-					pt.setX(p.x[k]);
-					pt.setY(p.y[k]);
-					poRing.setPoint(k, &pt);
-				}
-				if (poGeom.addRing(&poRing) != OGRERR_NONE ) {
-					message = "cannot add ring";
-					return false;
-				}
-
-				if (p.hasHoles()) {
-					for (size_t h=0; h < p.nHoles(); h++) {
-						SpatHole hole = p.getHole(h);
-						OGRLinearRing poHole;
-						for (size_t k=0; k<hole.size(); k++) {
-							pt.setX(hole.x[k]);
-							pt.setY(hole.y[k]);
-							poHole.setPoint(k, &pt);
-						}
-						if (poGeom.addRing(&poHole) != OGRERR_NONE ) {
-							message = "cannot add hole";
-							return false;
-						}
-					}
-				}
-				//closeRings
-			}
-			if (poFeature->SetGeometry( &poGeom ) != OGRERR_NONE) {
-				message = "cannot set geometry";
-				return false;
-			}
-		} else {
-			message = "Only points, lines and polygons are currently supported";
-			return false;
-		}
-
-		if( poLayer->CreateFeature( poFeature ) != OGRERR_NONE ) {
-			message = "Failed to create feature";
-			return false;
-        }
-
-        OGRFeature::DestroyFeature( poFeature );
-    }
-    //GDALClose( poDS );
-	//return true;
-	return poDS;
-}
-*/
-
-//#include "Rcpp.h"
-
-GDALDataset* SpatVector::write_ogr(std::string filename, std::string lyrname, std::string driver, bool overwrite, std::vector<std::string> options) {
-
+GDALDataset* SpatVector::write_ogr(std::string filename, std::string lyrname, std::string driver, bool append, bool overwrite, std::vector<std::string> options) {
 
     GDALDataset *poDS = NULL;
-
 	if (filename != "") {
-		if (file_exists(filename) & (!overwrite)) {
-			setError("file exists. Use 'overwrite=TRUE' to overwrite it");
-			return(poDS);
+		if (file_exists(filename)) {
+			if ((!overwrite) && (!append)) {
+				setError("file exists. Use 'overwrite=TRUE' to overwrite it");
+				return(poDS);
+			}
+		} else {
+			append = false;
 		}
 		if (nrow() == 0) {
 			setError("no geometries to write");
 			return(poDS);
 		}
 	}
-    GDALDriver *poDriver = GetGDALDriverManager()->GetDriverByName( driver.c_str() );
-    if( poDriver == NULL )  {
-        setError( driver + " driver not available");
-        return poDS;
-    }
-    char **papszMetadata;
-    papszMetadata = poDriver->GetMetadata();
-    if (!CSLFetchBoolean( papszMetadata, GDAL_DCAP_VECTOR, FALSE)) {
-		setError(driver + " is not a vector format");
-        return poDS;
+
+	if (append) {	
+	
+		#if GDAL_VERSION_MAJOR < 3
+			setError("GDAL >= 3 required for inserting layers into an existing file");
+			return(poDS);
+		#endif
+		
+		poDS = static_cast<GDALDataset*>(GDALOpenEx(filename.c_str(), GDAL_OF_VECTOR | GDAL_OF_UPDATE,
+				NULL, NULL, NULL ));
+
+		std::vector<std::string> lyrnms;
+		
+		size_t n = poDS->GetLayerCount();			
+		for (size_t i=0; i<n; i++) {
+			OGRLayer *poLayer = poDS->GetLayer(i);
+			if (poLayer != NULL) {
+				lyrnms.push_back((std::string)poLayer->GetName());
+			}
+		}
+		if (is_in_vector(lyrname, lyrnms)) {
+			if (!overwrite) {
+				setError("layer exists. Use 'overwrite=TRUE' to overwrite it");
+				return(poDS);
+			} else {
+				options.push_back("OVERWRITE=YES");
+			}
+		}
+	} else {
+		GDALDriver *poDriver = GetGDALDriverManager()->GetDriverByName( driver.c_str() );
+		if( poDriver == NULL )  {
+			setError( driver + " driver not available");
+			return poDS;
+		}
+		char **papszMetadata;
+		papszMetadata = poDriver->GetMetadata();
+		if (!CSLFetchBoolean( papszMetadata, GDAL_DCAP_VECTOR, FALSE)) {
+			setError(driver + " is not a vector format");
+			return poDS;
+		}
+		if (!CSLFetchBoolean( papszMetadata, GDAL_DCAP_CREATE, FALSE)) {
+			setError("cannot create a "+ driver + " dataset");
+			return poDS;
+		}	
+		poDS = poDriver->Create(filename.c_str(), 0, 0, 0, GDT_Unknown, NULL );
 	}
-    if (!CSLFetchBoolean( papszMetadata, GDAL_DCAP_CREATE, FALSE)) {
-		setError("cannot create a "+ driver + " dataset");
-        return poDS;
-	}
-    poDS = poDriver->Create(filename.c_str(), 0, 0, 0, GDT_Unknown, NULL );
+	
     if( poDS == NULL ) {
         setError("Creation of output dataset failed" );
         return poDS;
@@ -230,6 +108,7 @@ GDALDataset* SpatVector::write_ogr(std::string filename, std::string lyrname, st
 
 	std::string s = srs.wkt;
 
+
 	OGRSpatialReference *SRS = NULL;
 	if (s != "") {
 		SRS = new OGRSpatialReference;
@@ -244,13 +123,23 @@ GDALDataset* SpatVector::write_ogr(std::string filename, std::string lyrname, st
 		}
 	}
 
+	size_t nGroupTransactions = 0;
+
     OGRLayer *poLayer;
 	char** papszOptions = NULL;
 	if (options.size() > 0) {
 		for (size_t i=0; i<options.size(); i++) {
 			std::vector<std::string> gopt = strsplit(options[i], "=");
 			if (gopt.size() == 2) {
-				papszOptions = CSLSetNameValue(papszOptions, gopt[0].c_str(), gopt[1].c_str() );
+				if (gopt[0] == "nGroupTransactions") {
+					try  {
+						nGroupTransactions = std::stoi(gopt[1]);
+					} catch (std::invalid_argument &e)  {
+						nGroupTransactions = 0;
+					}
+				} else {
+					papszOptions = CSLSetNameValue(papszOptions, gopt[0].c_str(), gopt[1].c_str() );
+				}
 			}
 		}
 		// papszOptions = CSLSetNameValue( papszOptions, "ENCODING", "UTF-8" );
@@ -264,32 +153,56 @@ GDALDataset* SpatVector::write_ogr(std::string filename, std::string lyrname, st
 //	if (SRS != NULL) SRS->Release();
 	if (SRS != NULL) OSRDestroySpatialReference(SRS);
 
+
 	std::vector<std::string> nms = get_names();
 	std::vector<std::string> tps = df.get_datatypes();
 	OGRFieldType otype;
 	int nfields = nms.size();
 	size_t ngeoms = size();
 
+
 	for (int i=0; i<nfields; i++) {
+
+		OGRFieldSubType eSubType = OFSTNone;
 		if (tps[i] == "double") {
 			otype = OFTReal;
 		} else if (tps[i] == "long") {
 			otype = OFTInteger64;
+		} else if (tps[i] == "bool") {
+			otype = OFTInteger;
+			eSubType = OFSTBoolean;
 		} else {
 			otype = OFTString;
 		}
 
 		OGRFieldDefn oField(nms[i].c_str(), otype);
+		oField.SetSubType(eSubType);
 		if (otype == OFTString) {
 			oField.SetWidth(32); // needs to be computed
 		}
 		if( poLayer->CreateField( &oField ) != OGRERR_NONE ) {
 			setError( "Field creation failed for: " + nms[i]);
 			return poDS;
-		}
+		}		
 	}
 
-	//unsigned r = 0;
+	// use a single transaction as in sf
+	// makes a big difference for gpkg by avoiding many INSERTs	
+	bool can_do_transaction = poDS->TestCapability(ODsCTransactions); // == TRUE);
+	bool transaction = false;
+	if (can_do_transaction) { 
+		transaction = (poDS->StartTransaction() == OGRERR_NONE); 
+		if (! transaction) { 
+			setError("transaction failed");
+			return poDS; 
+		} 
+	}
+	// chunks
+	
+	if (nGroupTransactions == 0) {
+		nGroupTransactions = 50000;
+	}
+	size_t gcntr = 0;
 
 	for (size_t i=0; i<ngeoms; i++) {
 
@@ -300,6 +213,8 @@ GDALDataset* SpatVector::write_ogr(std::string filename, std::string lyrname, st
 				poFeature->SetField(j, df.getDvalue(i, j));
 			} else if (tps[j] == "long") {
 				poFeature->SetField(j, (GIntBig)df.getIvalue(i, j));
+			} else if (tps[j] == "bool") {
+				poFeature->SetField(j, df.getBvalue(i, j));
 			} else {
 				poFeature->SetField(j, df.getSvalue(i, j).c_str());
 			}
@@ -309,24 +224,21 @@ GDALDataset* SpatVector::write_ogr(std::string filename, std::string lyrname, st
 // points -- also need to do multi-points
 		OGRPoint pt;
 		if (wkb == wkbPoint) {
-			SpatGeom g = getGeom(i);
-			if (!std::isnan(g.parts[0].x[0])) {
-				pt.setX( g.parts[0].x[0] );
-				pt.setY( g.parts[0].y[0] );
+			if (!std::isnan(geoms[i].parts[0].x[0])) {
+				pt.setX( geoms[i].parts[0].x[0] );
+				pt.setY( geoms[i].parts[0].y[0] );
 			}
 			poFeature->SetGeometry( &pt );
 
 // lines
 		} else if (wkb == wkbMultiLineString) {
-			SpatGeom g = getGeom(i);
 			OGRMultiLineString poGeom;
-			for (size_t j=0; j<g.size(); j++) {
+			for (size_t j=0; j<geoms[i].size(); j++) {
 				OGRLineString poLine = OGRLineString();
-				SpatPart p = g.getPart(j);
-				for (size_t k=0; k<p.size(); k++) {
-					if (!std::isnan(p.x[k])) {
-						pt.setX(p.x[k]);
-						pt.setY(p.y[k]);
+				for (size_t k=0; k<geoms[i].parts[j].size(); k++) {
+					if (!std::isnan(geoms[i].parts[j].x[k])) {
+						pt.setX(geoms[i].parts[j].x[k]);
+						pt.setY(geoms[i].parts[j].y[k]);
 						poLine.setPoint(k, &pt);
 					}
 				}
@@ -393,19 +305,38 @@ GDALDataset* SpatVector::write_ogr(std::string filename, std::string lyrname, st
 			setError("Failed to create feature");
 			return poDS;
         }
-
         OGRFeature::DestroyFeature( poFeature );
+		gcntr++;
+		if (transaction && (gcntr == nGroupTransactions)) {
+			if (poDS->CommitTransaction() != OGRERR_NONE) {
+				poDS->RollbackTransaction();
+				setError("transaction commit failed");
+			}
+			gcntr = 0;
+			transaction = (poDS->StartTransaction() == OGRERR_NONE); 
+			if (! transaction) { 
+				setError("transaction failed");
+				return poDS; 
+			} 
+		}
     }
-    //GDALClose( poDS );
-	//return true;
+	if (transaction && (gcntr>0) && (poDS->CommitTransaction() != OGRERR_NONE)) {
+		poDS->RollbackTransaction();
+		setError("transaction commit failed");
+	} 
 	return poDS;
 }
 
 
 
-bool SpatVector::write(std::string filename, std::string lyrname, std::string driver, bool overwrite, std::vector<std::string> options) {
+bool SpatVector::write(std::string filename, std::string lyrname, std::string driver, bool append, bool overwrite, std::vector<std::string> options) {
 
-	GDALDataset *poDS = write_ogr(filename, lyrname, driver, overwrite, options);
+	if (nrow() == 0) {
+		addWarning("nothing to write");
+		return false;
+	}
+
+	GDALDataset *poDS = write_ogr(filename, lyrname, driver, append, overwrite, options);
     if (poDS != NULL) GDALClose( poDS );
 	if (hasError()) {
 		return false;
@@ -415,7 +346,7 @@ bool SpatVector::write(std::string filename, std::string lyrname, std::string dr
 }
 
 GDALDataset* SpatVector::GDAL_ds() {
-	return write_ogr("", "layer", "Memory", true, std::vector<std::string>());
+	return write_ogr("", "layer", "Memory", false, true, std::vector<std::string>());
 }
 
 
@@ -523,4 +454,68 @@ bool SpatDataFrame::write_dbf(std::string filename, bool overwrite, SpatOptions
 }
 
 
+bool SpatVector::delete_layers(std::string filename, std::vector<std::string> layers, bool return_error) {
+
+	if (filename == "") {
+		setError("empty filename");
+		return false;
+	}	
+	if (!file_exists(filename)) {
+		setError("file does not exist");
+		return false;
+	}
+	if (layers.size() == 0) return(true);
+
+    GDALDataset *poDS = static_cast<GDALDataset*>(GDALOpenEx(filename.c_str(), GDAL_OF_VECTOR | GDAL_OF_UPDATE,
+				NULL, NULL, NULL ));
+
+    if( poDS == NULL ) {
+        setError("Cannot open or update this dataset" );
+        return false;
+    }
+
+	std::string fails;
+	
+	size_t n = poDS->GetLayerCount();
+	for (int i =(n-1); i > 0; i--) {
+		size_t m = layers.size();
+		if (m == 0) break;
+		
+		OGRLayer *poLayer = poDS->GetLayer(i);
+		if (poLayer == NULL) continue;
+		std::string lname = poLayer->GetName();
+		for (size_t j=0; j<m; j++) {
+			if (lname.compare(layers[j]) == 0) {
+				OGRErr err = poDS->DeleteLayer(i);
+				if (err == OGRERR_UNSUPPORTED_OPERATION) {
+					setError("Deleting layer not supported for this file (format / driver)");
+					GDALClose(poDS);
+					return(false);
+				}
+				if (err != OGRERR_NONE) {
+					if (fails.size() > 0) {
+						fails += ", " + layers[j];
+					} else {
+						fails = layers[j];						
+					}
+				}
+				layers.erase(layers.begin() + j);
+				break;
+			}
+		}
+	}
+	GDALClose(poDS);
+	if (layers.size() > 0) {
+		fails += concatenate(layers, ", ");
+	}
+	if (fails.size() > 0) {
+		if (return_error) {
+			setError("deleting failed for: " + fails);
+		} else {
+			addWarning("deleting failed for: " + fails);
+		}
+	}
+	return true;
+}
+
 #endif
